C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C--MODULE NAME--
C                BUFALL.FOR
C--DESCRIPTION--
C                Contains the subprograms which call the fluid specific
C                routines used in calculating ancillary values as well
C                as the subroutine READREF which loads the EOS constants
C                into the common blocks.
C=======================================================================
C
      FUNCTION IDGAS(T, INDX)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the Ideal Gas Isobaric Heat Capacity (CP0/R), and
C    the integral functions (CP0/R dT) and (CP0/RT dT) as functions of
C    Temperature.  The value of INDX determines which function is
C    evaluated.  The integral functions can be used to calculate the
C    value of a definite integral by calling this subroutine at both
C    the upper and lower temperature bounds of the definite integral
C    using the appropriate value of INDX.
C
C    For the same fluid, temperature, and INDX, previously calculated
C    values saved in IDGVALQ(INDX) may be used.  IDGDONEQ(INDX) holds
C    flags that indicate if saved values can be used (.true.) or not
C    used (.false.).  When the fluid or temperature changes, all flags
C    are reset to .false.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T              Temperature in Kelvins. (Input)
C    INDX           A switch that specifies one of the three functions:
C                   If INDX = 0, calculate CP0/R
C                   If INDX = 1, calculate CP0/R dT
C                   If INDX = 2, calculate CP0/RT dT
C    IDGAS          Value of the function. (Output)
C    IDGDONEQ(0:2)  Logical array stored in COMMONS.FOR holding
C                   recalculation flags.
C    IDGVALQ(0:2)   Array of IDGAS values calculated for last
C                   T and NDX and stored in COMMONS.FOR.
C    TLASTQ         Last temperature, stored in COMMONS.FOR.
C    NDXLASTQ       Last NDX value, stored in COMMONS.FOR.
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER INDX
      REAL*8 T, IDGAS, IDGASX

C--Functions Called---------------------
      REAL*8 IDGHYD, IDGNH3

C--Local Variables----------------------
      INTEGER I
      REAL*8 A, B, C, CT, U, EXPU, TR, REDUC
C-----------------------------------------------------------------------
C  Test for proper values of the index.
C-----------------------------------------------------------------------
      IF (INDX.NE.1) THEN
        IF (INDX.NE.0) THEN
          IF (INDX.NE.2) THEN
            WRITE (*,*) ' *** IDGAS: INDX IS NOT 0, 1, 0R 2 ***'
            WRITE (ERRDEV,*) ' *** IDGAS: INDX IS NOT 0, 1, 0R 2 ***'
            STOP
          ENDIF
        ENDIF
      ENDIF

C  If temperature and fluid have not changed, see if function has
C  been saved.  If so, use it; if not, calculate and save it. If
C  temperature or fluid have changed, reset all flags to false.

      IF (T.EQ.TLASTQ .AND. NDX.EQ.NDXLASTQ) THEN
        IF (IDGDONEQ(INDX)) THEN
          IDGAS = IDGVALQ(INDX)
          RETURN
        ENDIF
      ELSE
        IDGDONEQ(0) = .FALSE.
        IDGDONEQ(1) = .FALSE.
        IDGDONEQ(2) = .FALSE.
      ENDIF
      IDGASX = 0
C-----------------------------------------------------------------------
C  Calculate Cp0/R.
C-----------------------------------------------------------------------
      REDUC = 1
      IF (CP0FLAGQ(NDX).EQ.3) REDUC = TC(NDX)
      IF (CP0FLAGQ(NDX).EQ.4) REDUC = TC(NDX)
      IF (CP0FLAGQ(NDX).EQ.5) REDUC = 100.0D0
      TR = T/REDUC
C.....The reduction flag has not been tested with the Einstein function.
      IF (CP0FLAGQ(NDX).GE.1 .AND. CP0FLAGQ(NDX).LE.6) THEN
        IF (INDX.EQ.0) THEN
          DO 10 I = 1, NTCP0Q(NDX)
            IF (PWRCP0Q(I, NDX).EQ.1000) THEN
              U = CP0COEF2Q(I, NDX)/T
              EXPU = DEXP(U)
              IDGASX = IDGASX+CP0COEFQ(I,NDX)*U*U*EXPU/(EXPU-1.D0)**2.D0
            ELSE
              IDGASX = IDGASX + CP0COEFQ(I, NDX)*TR**PWRCP0Q(I, NDX)
            ENDIF
 10       CONTINUE
C-----------------------------------------------------------------------
C  Calculate the definite integral function for Cp0/R dT.
C-----------------------------------------------------------------------
        ELSEIF (INDX.EQ.1) THEN
          DO 20 I = 1, NTCP0Q(NDX)
            IF (PWRCP0Q(I, NDX).EQ.1000) THEN
              U = CP0COEF2Q(I, NDX)/T
              EXPU = DEXP(U)
              IDGASX=IDGASX+CP0COEFQ(I,NDX)*CP0COEF2Q(I,NDX)/(EXPU-1.D0)
            ELSEIF (PWRCP0Q(I, NDX).EQ.-1) THEN
              IDGASX = IDGASX + CP0COEFQ(I, NDX)*DLOG(T)*REDUC
            ELSE
              IDGASX = IDGASX+CP0COEFQ(I,NDX)*TR**(PWRCP0Q(I,NDX)+1.D0)
     &                /(PWRCP0Q(I, NDX) + 1.D0)*REDUC
            ENDIF
 20       CONTINUE
C-----------------------------------------------------------------------
C  Calculate the definite integral function for Cp0/RT dT.
C-----------------------------------------------------------------------
        ELSEIF (INDX.EQ.2) THEN
          DO 30 I = 1, NTCP0Q(NDX)
            IF (PWRCP0Q(I, NDX).EQ.1000) THEN
              U = CP0COEF2Q(I, NDX)/T
              EXPU = DEXP(U)
              IDGASX = IDGASX + CP0COEFQ(I,NDX)*
     &                 (U*EXPU/(EXPU-1.D0)-DLOG(EXPU-1.D0))
            ELSEIF (PWRCP0Q(I, NDX).EQ.0) THEN
              IDGASX = IDGASX + CP0COEFQ(I, NDX)*DLOG(T)
            ELSE
              IDGASX = IDGASX + CP0COEFQ(I, NDX)*TR**PWRCP0Q(I, NDX)/
     &                PWRCP0Q(I, NDX)
            ENDIF
 30       CONTINUE
        ENDIF
C.......Equation Type 2 is used for dimensional equations in J/mol-K.
        IF (CP0FLAGQ(NDX).EQ.2) IDGASX = IDGASX/RGAS(NDX)/1000.D0
C.......Equation Type 4 is used for J/kg-K units and T is reduced by Tc.
        IF (CP0FLAGQ(NDX).EQ.4) IDGASX=IDGASX/RGAS(NDX)*AMOL(NDX)/1.D6
C.......Equation Type 6 is used for kJ/kg-K units
        IF (CP0FLAGQ(NDX).EQ.6) IDGASX=IDGASX/RGAS(NDX)*AMOL(NDX)/1.D3
C-----------------------------------------------------------------------
      ELSEIF (CP0FLAGQ(NDX).EQ.30) THEN
        A = CP0COEFQ(1, NDX)
        B = CP0COEFQ(2, NDX)
        C = CP0COEFQ(3, NDX)
        CT = 1 + C*T
        IF (INDX.EQ.0) THEN
          IDGASX = (A + B*T)/CT
        ELSEIF (INDX.EQ.1) THEN
          IDGASX = (A/C - B/C**2.D0)*DLOG(CT) + B/C**2.D0*CT
        ELSEIF (INDX.EQ.2) THEN
          IDGASX = B/C*DLOG(CT) - A*DLOG(CT/T)
        ENDIF
      ELSEIF (CP0FLAGQ(NDX).EQ.31) THEN
        IDGASX = IDGNH3(T, INDX)
      ELSEIF (CP0FLAGQ(NDX).EQ.32) THEN
        IDGASX = IDGHYD(T, INDX)
      ELSE
        WRITE (*,100) TNAMEQ(NDX)
        WRITE (ERRDEV, 100) TNAMEQ(NDX)
 100    FORMAT (' *** IDGAS: Old ',A8,' reference file detected:',
     &          ' v 3.0 or above is required***')
        STOP
      ENDIF

C-----------------------------------------------------------------------
C  Save current values and set flag.
      TLASTQ = T
      NDXLASTQ = NDX
      IDGVALQ(INDX) = IDGASX
      IDGDONEQ(INDX) = .TRUE.
      IDGAS = IDGASX

      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION P2PEQ(T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Buffer call to estimate the vapor pressure for a fluid.  This
C    routine calculates saturated vapor pressure when IPHASE = 1 or
C    IPHASE = 2 and melting pressure when IPHASE = 3.
C
C    For a mixture such as air, IPHASE = 1 specifies a bubble point state,
C    IPHASE = 2 specifies a dew point state and IPHASE = 3 specifies
C    a freezing point state.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T       Saturation Temperature in Kelvins. (Input)
C    IPHASE  Switch to select liquid, vapor or melting phase. (Input)
C    P2PEQ   Vapor pressure in MPa. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 T, P2PEQ, P2PEQX

C--Functions Called---------------------
      REAL*8 P2PAIR

C--Local Variables----------------------
      REAL*8 THETA, A8, A9, A10
      INTEGER I

C-----------------------------------------------------------------------
C  Check the temperature bounds.
C-----------------------------------------------------------------------
      IF (T.GT.TC(NDX) .AND. IPHASE.NE.3) THEN
        WRITE (*,    1000) T, TNAMEQ(NDX)
        IF (IOUT.NE.0) WRITE (IOUT, 1000) T, TNAMEQ(NDX)
        RETURN
      ELSEIF (T.LT.TTRP(NDX)-1.1D0) THEN
        WRITE (*,    1010) T, TNAMEQ(NDX)
        IF (IOUT.NE.0) WRITE (IOUT, 1010) T, TNAMEQ(NDX)
      ENDIF
 1000 FORMAT(' *** P2PEQ: TEMPERATURE',F12.6,' > TC for ',A8,' ***')
 1010 FORMAT(' *** P2PEQ: TEMPERATURE',F12.6,' < TTRP for ',A8,' ***')

C-----------------------------------------------------------------------
C  Calculate vapor pressure.
C-----------------------------------------------------------------------
      P2PEQX = 0
      IF (PVFLAGQ(NDX).EQ.31 .AND. IPHASE.NE.3) THEN
        P2PEQX = P2PAIR(T, IPHASE)
      ELSEIF (T.EQ.TC(NDX) .AND. IPHASE.NE.3) THEN
        P2PEQX = PC(NDX)
      ELSEIF (IPHASE.EQ.1 .OR. IPHASE.EQ.2) THEN
        IF (PVFLAGQ(NDX).GE.1 .AND. PVFLAGQ(NDX).LE.6) THEN
          IF (PVFLAGQ(NDX).EQ.1) THETA = DSQRT(1.D0 - T/TC(NDX))
          IF (PVFLAGQ(NDX).EQ.5) THETA = 1.D0 - T/TC(NDX)
          IF (PVFLAGQ(NDX).EQ.2 .OR. PVFLAGQ(NDX).EQ.3)
     &       THETA = DSQRT(TC(NDX)/T - 1.D0)
          IF (PVFLAGQ(NDX).EQ.4) THETA = T
          IF (PVFLAGQ(NDX).EQ.6) THETA = T/TC(NDX)
          DO 100 I=1, NTPVQ(NDX)
            IF (PWRPVQ(I, NDX).LT.1000) THEN
              P2PEQX = P2PEQX + PVCOEFQ(I, NDX)*THETA**PWRPVQ(I, NDX)
            ELSEIF (PWRPVQ(I, NDX).EQ.1000) THEN
              P2PEQX = P2PEQX + PVCOEFQ(I, NDX)*DLOG(THETA)
            ELSEIF (PWRPVQ(I, NDX).EQ.1001) THEN
              P2PEQX = P2PEQX+PVCOEFQ(I, NDX)*(1.D0-T/TC(NDX))**1.935D0
            ELSEIF (PWRPVQ(I, NDX).EQ.1002) THEN
              P2PEQX = P2PEQX + PVCOEFQ(I, NDX)*THETA/(1 - THETA)
            ELSEIF (PWRPVQ(I, NDX).EQ.1003) THEN
              P2PEQX = P2PEQX + PVCOEFQ(I, NDX)*(1.D0-T/TC(NDX))**1.9D0
            ENDIF
 100      CONTINUE
          IF (PVFLAGQ(NDX).EQ.1) P2PEQX = PC(NDX)*DEXP(TC(NDX)/T*P2PEQX)
          IF (PVFLAGQ(NDX).EQ.2) P2PEQX = PC(NDX)*DEXP(P2PEQX)
          IF (PVFLAGQ(NDX).EQ.5) P2PEQX = PC(NDX)*DEXP(P2PEQX)
          IF (PVFLAGQ(NDX).EQ.3) P2PEQX = PC(NDX)*T/TC(NDX)*P2PEQX
          IF (PVFLAGQ(NDX).EQ.4) P2PEQX = DEXP(P2PEQX)
          IF (PVFLAGQ(NDX).EQ.6) P2PEQX = DEXP(P2PEQX)/1000000.
        ELSEIF(PVFLAGQ(NDX).EQ.20) THEN
          THETA = (1 - TTRP(NDX)/T) / (1 - TTRP(NDX)/TC(NDX))
          DO 110 I=1, NTPVQ(NDX)
            IF (PWRPVQ(I, NDX).LT.1000) THEN
              P2PEQX = P2PEQX + PVCOEFQ(I, NDX)*THETA**PWRPVQ(I, NDX)
            ELSEIF (PWRPVQ(I, NDX).EQ.1000) THEN
              P2PEQX = P2PEQX+PVCOEFQ(I, NDX)*THETA*(1.D0-THETA)**1.6D0
            ELSEIF (PWRPVQ(I, NDX).EQ.1001) THEN
              P2PEQX = P2PEQX+PVCOEFQ(I, NDX)*THETA*(1.D0-THETA)**1.2D0
            ELSEIF (PWRPVQ(I, NDX).EQ.1002) THEN
              P2PEQX = P2PEQX+PVCOEFQ(I, NDX)*THETA*(1.D0-THETA)**1.03D0
            ELSEIF (PWRPVQ(I, NDX).EQ.1003) THEN
              P2PEQX = P2PEQX+PVCOEFQ(I, NDX)*THETA*(1.D0-THETA)**1.4D0
            ELSEIF (PWRPVQ(I, NDX).EQ.1004) THEN
              P2PEQX = P2PEQX+PVCOEFQ(I, NDX)*THETA*(1.D0-THETA)**1.5D0
            ENDIF
 110      CONTINUE
          P2PEQX = DEXP(DLOG(PTRP(NDX)) + P2PEQX)
        ELSEIF(PVFLAGQ(NDX).EQ.21) THEN
          A8 = 0
          A9 = 0
          A10 = 0
          DO 130 I=1, NTPVQ(NDX)
            IF (PWRPVQ(I, NDX).EQ.1000) THEN
              P2PEQX = P2PEQX + PVCOEFQ(I, NDX)*DLOG10(T)
            ELSEIF (PWRPVQ(I, NDX).EQ.1001) THEN
              A8 = PVCOEFQ(I, NDX)
            ELSEIF (PWRPVQ(I, NDX).EQ.1002) THEN
              A9 = PVCOEFQ(I, NDX)
            ELSEIF (PWRPVQ(I, NDX).EQ.1003) THEN
              A10 = PVCOEFQ(I, NDX)
            ELSE
              P2PEQX = P2PEQX + PVCOEFQ(I, NDX)*T**PWRPVQ(I, NDX)
            ENDIF
 130      CONTINUE
          IF (A8.NE.0) P2PEQX = P2PEQX + A8*(A9-T)/T*DLOG10((A9-T)*A10)
          P2PEQX = 10**P2PEQX
        ENDIF

C-----------------------------------------------------------------------
C  Calculate melting point pressure.
C-----------------------------------------------------------------------
      ELSEIF (IPHASE.EQ.3) THEN
        IF (PMFLAGQ(NDX).EQ.1) THEN
          THETA = T/TTRP(NDX) - 1.D0
          DO 150 I=1, NTPMQ(NDX)
 150      P2PEQX = P2PEQX + PMCOEFQ(I, NDX)*THETA**PWRPMQ(I, NDX)
          P2PEQX = PTRP(NDX)*DEXP(P2PEQX)
        ELSEIF (PMFLAGQ(NDX).EQ.2) THEN
          THETA = T/TTRP(NDX)
          P2PEQX = 1.D0
     &          + PMCOEFQ(1, NDX)*(THETA**PMCOEFQ(2, NDX) - 1.D0)
     &          + PMCOEFQ(3, NDX)*(THETA**PMCOEFQ(4, NDX) - 1.D0)
          P2PEQX = PTRP(NDX)*P2PEQX
        ELSEIF (PMFLAGQ(NDX).EQ.3) THEN
          P2PEQX = PTRP(NDX)
     &          + PMCOEFQ(1, NDX)*((T/TTRP(NDX))**PMCOEFQ(2, NDX)-1.D0)
        ELSEIF (PMFLAGQ(NDX).EQ.4) THEN
          THETA = (T/TTRP(NDX))**3.D0 - 1.D0
          P2PEQX = PTRP(NDX)*(1.D0 + PMCOEFQ(1, NDX)*THETA)
        ELSEIF (PMFLAGQ(NDX).EQ.5) THEN
          P2PEQX = (PMCOEFQ(1, NDX)*(T+PMCOEFQ(4, NDX))**PMCOEFQ(3, NDX)
     &          +  PMCOEFQ(2, NDX))/10.D0
        ELSEIF (PMFLAGQ(NDX).EQ.6) THEN
          P2PEQX = DLOG(T/PMCOEFQ(1, NDX))*PMCOEFQ(2, NDX)
        ELSEIF (PMFLAGQ(NDX).EQ.20) THEN
          P2PEQX = 0.D0
          DO 120 I=1, NTPMQ(NDX)
 120      P2PEQX = P2PEQX + PMCOEFQ(I, NDX)*T**PWRPMQ(I, NDX)
        ELSEIF (PMFLAGQ(NDX).EQ.30) THEN
          IF (T.GT.22) THEN
            P2PEQX = PMCOEFQ(1, NDX) + PMCOEFQ(2, NDX)*T**PMCOEFQ(3,NDX)
          ELSE
            P2PEQX = PMCOEFQ(4, NDX) + PMCOEFQ(5, NDX)*T**PMCOEFQ(6,NDX)
          ENDIF
        ELSEIF (PMFLAGQ(NDX).EQ.31) THEN
C  Added fix for area between ttrp (279.47) and 279.7, since equation
C  would be negative in that region.  may want to refit melting data. (cyclohex)
          IF (T.LE.279.75D0) THEN
            P2PEQX = .3259071D0 * (T - 279.47D0) + PTRP(NDX)
          ELSE
            P2PEQX = 383.4D0 * ( (T/279.7D0)**1.41D0 - 1.D0)
          ENDIF
        ELSE
          P2PEQX = PHIGH(NDX)
        ENDIF
      ELSE
        WRITE (*, 10) TNAMEQ(NDX)
        WRITE (ERRDEV, 10) TNAMEQ(NDX)
 10     FORMAT (' *** P2PEQX: Old ',A8,' reference file detected:',
     &            ' v 3.0 or above is required***')
        STOP
      ENDIF
      P2PEQ = P2PEQX
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION D2PEQ(T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Buffer call to estimate coexistence densities for a fluid.  This
C    routine calculates saturated liquid density when IPHASE = 1,
C    saturated vapor density when IPHASE = 2 and freezing density
C    when IPHASE = 3.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T       Saturation Temperature in Kelvins. (Input)
C    IPHASE  Switch to select liquid, vapor or melting phase. (Input)
C    D2PEQ   Coexistence density in mol/dm3. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 T, D2PEQ, D2PEQX

C--Functions Called---------------------
      REAL*8 DOFT3, P2PEQ

C--Local Variables----------------------
      REAL*8 DLTRP, DVTRP, THETA, DENOM, ZC, PSAT, TPWR
      INTEGER I

C-----------------------------------------------------------------------
C  Check the temperature bounds.
C-----------------------------------------------------------------------
      IF (IPHASE.EQ.3) THEN
        D2PEQ = DOFT3(T, IPHASE)
        RETURN
      ELSEIF (T.GT.TC(NDX)) THEN
        WRITE (*,    1000) T, TNAMEQ(NDX)
        IF (IOUT.NE.0) WRITE (IOUT, 1000) T, TNAMEQ(NDX)
        RETURN
      ELSEIF (T.LT.TTRP(NDX)-1.1D0) THEN
        WRITE (*,    1010) T, TNAMEQ(NDX)
        IF (IOUT.NE.0) WRITE (IOUT, 1010) T, TNAMEQ(NDX)
      ENDIF
 1000 FORMAT(' *** D2PEQ: TEMPERATURE',F12.6,' > TC for ',A8,' ***')
 1010 FORMAT(' *** D2PEQ: TEMPERATURE',F12.6,' < TTRP for ',A8,' ***')
      D2PEQX = 0
      IF (T.EQ.TC(NDX)) THEN
        D2PEQX = DC(NDX)
C-----------------------------------------------------------------------
C  Calculate saturated liquid density.
C-----------------------------------------------------------------------
      ELSEIF (DLFLAGQ(NDX).GE.1 .AND. DLFLAGQ(NDX).LE.4
     &       .AND. IPHASE.EQ.1) THEN
        IF (DLFLAGQ(NDX).NE.2) THETA = (1.D0 - T/TC(NDX))**(1.D0/3.D0)
        IF (DLFLAGQ(NDX).EQ.2) THETA = (TC(NDX)/T - 1.D0)**(1.D0/3.D0)
        IF (DLFLAGQ(NDX).EQ.4) THETA = 1.D0 - T/TC(NDX)
        DENOM = 1
        DO 100 I=1, NTDLQ(NDX)
          IF (PWRDLQ(I, NDX).LT.1000) THEN
            D2PEQX = D2PEQX + DLCOEFQ(I, NDX)*THETA**PWRDLQ(I, NDX)
          ELSEIF (PWRDLQ(I, NDX).EQ.1000) THEN
            D2PEQX = D2PEQX + DLCOEFQ(I, NDX)*DLOG(T/TC(NDX))
          ELSEIF (PWRDLQ(I, NDX).EQ.1001) THEN
            DENOM = 1 + DLCOEFQ(I, NDX)*THETA**(1.D0 - 0.355D0)
          ELSEIF (PWRDLQ(I, NDX).EQ.1002) THEN
            DENOM = 1 + DLCOEFQ(I, NDX)*THETA**(1.D0 - 0.35D0)
          ENDIF
 100    CONTINUE
        D2PEQX = D2PEQX/DENOM
        IF (DLFLAGQ(NDX).NE.3) D2PEQX = DC(NDX)*(D2PEQX + 1.D0)
        IF (DLFLAGQ(NDX).EQ.3) D2PEQX = DC(NDX)*DEXP(D2PEQX)
C-----------------------------------------------------------------------
C  Calculate saturated vapor density.
C-----------------------------------------------------------------------
      ELSEIF (DVFLAGQ(NDX).GE.1 .AND. DVFLAGQ(NDX).LE.4
     &        .AND. IPHASE.EQ.2) THEN
        IF (DVFLAGQ(NDX).NE.2) THETA = (1.D0 - T/TC(NDX))**(1.D0/3.D0)
        IF (DVFLAGQ(NDX).EQ.2) THETA = (TC(NDX)/T - 1.D0)**(1.D0/3.D0)
        IF (DVFLAGQ(NDX).EQ.4) THETA = 1.D0 - T/TC(NDX)
        DO 110 I=1, NTDVQ(NDX)
          IF (PWRDVQ(I, NDX).LT.1000) THEN
            D2PEQX = D2PEQX + DVCOEFQ(I, NDX)*THETA**PWRDVQ(I, NDX)
          ELSEIF (PWRDVQ(I, NDX).EQ.1000) THEN
            D2PEQX = D2PEQX + DVCOEFQ(I, NDX)*DLOG(T/TC(NDX))
          ENDIF
 110    CONTINUE
        IF (DVFLAGQ(NDX).NE.3) D2PEQX = DC(NDX)*DEXP(D2PEQX)
        IF (DVFLAGQ(NDX).EQ.3) D2PEQX = DC(NDX)*(D2PEQX + 1.D0)
C-----------------------------------------------------------------------
      ELSEIF (DLFLAGQ(NDX).EQ.4 .AND. IPHASE.EQ.1) THEN
        DO 115 I=1,NTDLQ(NDX)
 115    D2PEQX = D2PEQX + DLCOEFQ(I,NDX)*T**(I-1)
C-----------------------------------------------------------------------
      ELSEIF (DLFLAGQ(NDX).EQ.20 .AND. IPHASE.EQ.1) THEN
        THETA = ((TC(NDX) - T)/(TC(NDX) - TTRP(NDX)))**(1.D0/3.D0)
        DO 120 I=1, NTDLQ(NDX)
          IF (PWRDLQ(I, NDX).LT.1000) THEN
            D2PEQX = D2PEQX + DLCOEFQ(I, NDX)*(1 - THETA**PWRDLQ(I,NDX))
          ELSEIF (PWRDLQ(I, NDX).EQ.1000) THEN
            D2PEQX = D2PEQX + DLCOEFQ(I, NDX)*DLOG(THETA**3.D0)
          ELSEIF (PWRDLQ(I, NDX).EQ.1001) THEN
            D2PEQX = D2PEQX + DLCOEFQ(I, NDX)*DLOG(THETA**(-3.D0))
          ELSEIF (PWRDLQ(I, NDX).EQ.1100) THEN
            DLTRP = DLCOEFQ(I, NDX)
          ENDIF
 120    CONTINUE
        D2PEQX = DC(NDX) + (DLTRP - DC(NDX)) * DEXP(D2PEQX)
      ELSEIF ((DVFLAGQ(NDX).EQ.20 .OR. DVFLAGQ(NDX).EQ.22)
     &        .AND. IPHASE.EQ.2) THEN
        THETA = ((TC(NDX) - T)/(TC(NDX) - TTRP(NDX)))**(1.D0/3.D0)
        DO 130 I=1, NTDVQ(NDX)
          IF (PWRDVQ(I, NDX).LT.1000) THEN
            IF (DVFLAGQ(NDX).EQ.20) THEN
              D2PEQX = D2PEQX + DVCOEFQ(I, NDX)*THETA**PWRDVQ(I, NDX)
            ELSEIF (DVFLAGQ(NDX).EQ.22) THEN
              D2PEQX = D2PEQX + DVCOEFQ(I, NDX)*(1-THETA**PWRDVQ(I,NDX))
            ENDIF
          ELSEIF (PWRDVQ(I, NDX).EQ.1000) THEN
            D2PEQX = D2PEQX + DVCOEFQ(I, NDX)*TC(NDX)*THETA**3.D0/T
          ELSEIF (PWRDVQ(I, NDX).EQ.1001) THEN
            D2PEQX = D2PEQX + DVCOEFQ(I, NDX)*DLOG(THETA**(-3.D0))
          ELSEIF (PWRDVQ(I, NDX).EQ.1100) THEN
            DVTRP = DVCOEFQ(I, NDX)
          ENDIF
 130    CONTINUE
        IF (DVFLAGQ(NDX).EQ.20) THEN
          D2PEQX = DEXP(DLOG(DC(NDX)) - DLOG(DC(NDX)/DVTRP)*D2PEQX)
        ELSEIF (DVFLAGQ(NDX).EQ.22) THEN
          D2PEQX = DC(NDX) + (DVTRP - DC(NDX)) * DEXP(D2PEQX)
        ENDIF
C-----------------------------------------------------------------------
      ELSEIF (DVFLAGQ(NDX).EQ.21 .AND. IPHASE.EQ.2) THEN
        THETA = 1.D0 - T/TC(NDX)
        DENOM = 1
        TPWR = 1
        DO 140 I=1, NTDVQ(NDX)
          IF (PWRDVQ(I, NDX).LT.1000) THEN
            D2PEQX = D2PEQX + DVCOEFQ(I, NDX)*THETA**PWRDVQ(I, NDX)
          ELSEIF (PWRDVQ(I, NDX).EQ.1001) THEN
            DENOM = 1 + DVCOEFQ(I, NDX)*THETA
          ELSEIF (PWRDVQ(I, NDX).EQ.1002) THEN
            TPWR = DVCOEFQ(I, NDX)
          ENDIF
 140    CONTINUE
        D2PEQX = D2PEQX/DENOM
        ZC = PC(NDX)/TC(NDX)/DC(NDX)/RGAS(NDX)
        PSAT = P2PEQ(T, 1)
        D2PEQX = PSAT/RGAS(NDX)/T/
     &         (1 + PSAT/PC(NDX) * ((TC(NDX)/T)**TPWR) * (ZC - 1)
     &        *(1 + D2PEQX))
      ELSE
        WRITE (*, 10) TNAMEQ(NDX)
        WRITE (ERRDEV, 10) TNAMEQ(NDX)
 10     FORMAT (' *** D2PEQ: Old ',A8,' reference file detected:',
     &            ' v 3.0 or above is required***')
        STOP
      ENDIF
      D2PEQ = D2PEQX
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION IDGNH3(T,INDX)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the Ideal Gas Isobaric Heat Capacity (CP0/R), and
C    the integral functions (CP0/R dT) and (CP0/RT dT) for
C    Ammonia as functions of Temperature.  The value of INDX determines
C    which function is evaluated.  The integral functions can be used
C    to calculate the value of a definite integral by calling this
C    subroutine at both the upper and lower temperature bounds of the
C    definite integral using the appropriate value of INDX.
C
C  REFERENCE:
C
C     Haar, L. and Gallagher, J.S., "Thermodynamic Properties of Ammonia"
C
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T      TEMPERATURE IN KELVINS (INPUT)
C    INDX   A SWITCH THAT SPECIFIES ONE OF THE THREE FUNCTIONS
C           IF INDX = 0, CALCULATE CP0/R
C           IF INDX = 1, CALCULATE CP0/R DT
C           IF INDX = 2, CALCULATE CP0/RT DT
C    IDGMET VALUE OF THE FUNCTION (OUTPUT)
C-----------------------------------------------------------------------

C--Arguments----------------------------
      INTEGER INDX
      REAL*8 T, IDGNH3, IDGNH3X

C--Local Variables----------------------
      INTEGER I
      REAL*8 N(11)

C-----------------------------------------------------------------------
      N(1)  =  0.38727270D+01
      N(2)  = -0.64463724D+00
      N(3)  = -0.32238759D+01
      N(4)  =  0.21376925D-02
      N(5)  = -0.86890833D-05
      N(6)  =  0.24085149D-07
      N(7)  = -0.36893175D-10
      N(8)  =  0.35034664D-13
      N(9)  = -0.20563027D-16
      N(10) =  0.68534200D-20
      N(11) = -0.99392427D-24
C-----------------------------------------------------------------------
C  Calculate Cp0/R.
C-----------------------------------------------------------------------
      IF (INDX.EQ.0) THEN
        IDGNH3X = N(1)
        DO 10 I=2, 11
 10     IDGNH3X = IDGNH3X + (I - 2)*(I - 3)*N(I)*T**(I - 3)
C-----------------------------------------------------------------------
C  Calculate the definite integral function for Cp0/R dT.
C-----------------------------------------------------------------------
      ELSEIF (INDX.EQ.1) THEN
        IDGNH3X = N(1)*T
        DO 20 I=2, 11
 20     IDGNH3X = IDGNH3X + (I - 3)*N(I)*T**(I - 2)
C-----------------------------------------------------------------------
C  Calculate the definite integral function for Cp0/RT dT.
C-----------------------------------------------------------------------
      ELSEIF (INDX.EQ.2) THEN
        IDGNH3X = N(1)*DLOG(T)
        DO 30 I=2, 11
 30     IDGNH3X = IDGNH3X + (I - 2)*N(I)*T**(I - 3)
      ENDIF
      IDGNH3 = IDGNH3X
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION IDGHYD(T,INDX)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C    Calculates the Ideal Gas Isobaric Heat Capacity (CP0/R) and the
C    the integral functions (CP0/R dT) and (CP0/RT dT) for parahydrogen
C    as functions of Temperature.  The value of INDX determines which
C    function is evaluated.
C
C    Equations and coefficients are taken from J Phys Chem, Vol 11
C    by B.A. Younglove (1982).
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T      Temperature in Kelvins (Input)
C    INDX   A switch that specifies one of the three functions
C           If INDX = 0, calculate CP0/R
C           If INDX = 1, calculate CP0/R dT
C           If INDX = 2, calculate CP0/RT dT
C    IDGHYD Value of the function (Output)
C-----------------------------------------------------------------------

C--Arguments----------------------------
      INTEGER INDX
      REAL*8 T, IDGHYD, IDGHYDX

C--Local Variables----------------------
      REAL*8 G1, G2, G3, G4, G5, G6, G7, G8, G9, C2, C1, U, EXPU

C-----------------------------------------------------------------------
      IF (T.GT.140) THEN
        G1 =  0.5262185164597D8
        G2 = -0.1487906248823D7
        G3 =  0.1601391392264D5
        G4 = -0.8031235938946D2
        G5 =  0.2307407941873D0
        G6 = -0.3176386248370D-3
        G7 =  0.1643857271214D-6
        G8 =  0.9230816464058D1
        G9 =  0.3D4
        C2 =  0.0D0
        C1 =  0.0D0
      ELSEIF (T.GT.40) THEN
        G1 =  0.2905965792270D6
        G2 = -0.2831103639248D5
        G3 =  0.1050424877391D4
        G4 = -0.1535751501769D2
        G5 =  0.1218941696566D0
        G6 = -0.2599406479908D-4
        G7 = -0.1288757333406D-5
        G8 =  0.1717441975231D6
        G9 =  0.3D4
        C2 = -383.280510075D0
        C1 =  74880.4778693D0
      ELSE
        G1 = 0.D0
        G2 = 0.D0
        G3 = 0.D0
        G4 = 2.50003150D0
        G5 = 0.D0
        G6 = 0.D0
        G7 = 0.D0
        G8 = 0.D0
        G9 = 0.D0
        C2 = -463.2543204110D0
        C1 = 78754.1672267D0
      ENDIF
      U= G9/T
      EXPU=DEXP(U)

C-----------------------------------------------------------------------
C  Calculate Cp0/R.
C-----------------------------------------------------------------------
      IF (INDX.EQ.0) THEN
        IF (T.LE.40) THEN
          IDGHYDX = G4
        ELSE
          IDGHYDX =  G1/T**3.D0 + G2/T**2.D0 + G3/T
     &           + G4 + G5*T + G6*T**2.D0 + G7*T**3.D0
          IDGHYDX = IDGHYDX + G8*U*U*EXPU / (EXPU-1.0D0)**2.D0
        ENDIF

C-----------------------------------------------------------------------
C  Calculate the definite integral function for Cp0/R dT.
C-----------------------------------------------------------------------
      ELSE IF (INDX.EQ.1) THEN
        IF (T.LE.40) THEN
          IDGHYDX = G4*T + C1
        ELSE
          IDGHYDX = -(G1/(T**2.D0)/2.D0) - G2/T + G3*DLOG(T)
     &           + G4*T + G5*(T**2.D0)/2.D0 + G6*(T**3.D0)/3.D0
     &           + G7*(T**4.D0)/4.D0 + C1
          IDGHYDX = IDGHYDX  - G8*G9/(1.0D0-EXPU)
        ENDIF

C-----------------------------------------------------------------------
C  Calculate the definite integral function for Cp0/RT dT.
C-----------------------------------------------------------------------
      ELSE IF (INDX.EQ.2) THEN
        IF (T.LE.40) THEN
          IDGHYDX = G4*DLOG(T) + C2
        ELSE
          IDGHYDX = -(G1/(T**3.D0)/3.D0) - G2/(T**2.D0)/2.D0
     &           - G3/T + G4*DLOG(T) + G5*T
     &           + G6*(T**2.D0)/2.D0 + G7*(T**3.D0)/3.D0 + C2
          IDGHYDX = IDGHYDX - G8*(U/(DEXP(-U)-1.0D0)+DLOG(EXPU-1.0D0))
        ENDIF
      ENDIF
      IDGHYD = IDGHYDX
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION P2PAIR(T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the Pressure for coexistence states of Air given
C    Temperature.
C            IPHASE = 1, specifies a bubble line state.
C            IPHASE = 2, specifies a dew line state.
C            IPHASE = 3, specifies a melting line state.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T       Temperature in Kelvins (Input)
C    IPHASE  Switch to select the bubble, dew, or melting lines. (Input)
C    P2PAIR  Vapor Pressure in MPa (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 T, P2PAIR, P2PAIRX

C--Local Variables----------------------
      REAL*8 THETA

C-----------------------------------------------------------------------
C  If IPHASE = 1, calculate the bubble point pressure of air
C-----------------------------------------------------------------------
      IF (IPHASE.EQ.1) THEN
        THETA = DSQRT(1.D0 - T/TC(NDX))
        P2PAIRX = 0.2095592444D+00*THETA
     &         - 0.6654905539D+01*THETA**2.D0
     &         + 0.2213718815D+02*THETA**4.D0
     &         - 0.8414553609D+02*THETA**5.D0
     &         + 0.1359753732D+03*THETA**6.D0
     &         - 0.8366895082D+02*THETA**7.D0
     &         + 0.1797856602D+02*THETA**12.D0
        P2PAIRX = PC(NDX)*DEXP(P2PAIRX*TC(NDX)/T)
C-----------------------------------------------------------------------
C  If IPHASE = 2, calculate the dew point pressure of air
C-----------------------------------------------------------------------
      ELSEIF (IPHASE.EQ.2) THEN
        THETA = DSQRT(1.D0 - T/TC(NDX))
        P2PAIRX =-0.1537763029D+00*THETA
     &         - 0.5544542064D+01*THETA**2.D0
     &         + 0.3127182733D+03*THETA**10.D0
     &         - 0.8959553274D+03*THETA**11.D0
     &         + 0.1834176566D+04*THETA**13.D0
     &         - 0.1321892808D+04*THETA**14.D0
        P2PAIRX = PC(NDX)*DEXP(P2PAIRX*TC(NDX)/T)
C-----------------------------------------------------------------------
C  Improper phase specification.
C-----------------------------------------------------------------------
      ELSE
        WRITE (*,*) ' *** P2PAIR: IPHASE IS NOT 1, 2, OR 3 ***'
        WRITE (ERRDEV,*) ' *** P2PAIR: IPHASE IS NOT 1, 2, OR 3 ***'
        STOP
      ENDIF
      P2PAIR = P2PAIRX
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE NEXTREC(FILEN, RECORD)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    This subroutine returns the next 80 character record from the
C    file associated with the file number, FILEN.  Blank records
C    and comment lines designated by a "#" are ignored.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      INTEGER FILEN
      CHARACTER*80 RECORD

C-----------------------------------------------------------------------
 15   READ (FILEN, '(A80)', END = 9999) RECORD
      IF (RECORD.EQ.' ') GO TO 15
      IF (RECORD(1:1).EQ.'#') GO TO 15
      RETURN
 9999 STOP ' NEXTREC: Unexpected End of File.'
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE READREF(FXNAME, INDX)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C    This routine initializes variables in COMMON blocks.
C
C  DESCRIPTION OF ARGUMENTS:
C    FXNAME  - Filespec for the fluid reference file (Input)
C    INDX    - Integer constant that specifies the storage location
C              of the coefficients.  INDX may have a value from 1 to
C              the parameter NF.  All arrays initialized by this
C              reference routine are dimensioned to accomodate multiple
C              fluids if NF > 1. INDX determines which column of these
C              arrays contain the constants for the specified fluid.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      CHARACTER FXNAME*(*)
      INTEGER INDX

C--Local Variables----------------------
      LOGICAL IDGTEMP
      CHARACTER*80 RECORD
      INTEGER K, FILEN, FLAG, SPMETH
      REAL*8 IDGAS

      FLAG = 0
      IF (FXNAME(1:1).EQ.'-') THEN
        FXNAME = FXNAME(2:LEN(FXNAME))
        FLAG = 1
      ENDIF
      IF (FXNAME(1:2).EQ.'R-' .OR. FXNAME(1:2).EQ.'r-') THEN
        FXNAME = FXNAME(1:1)//FXNAME(3:LEN(FXNAME))
      ENDIF
      IF (INDX.LT.0) THEN
        INDX = ABS(INDX)
        FLAG = 1
      ENDIF
      FILEN = 91
      IF (INDX.GT.NF) STOP '*** READREF: INDX > NUMBER OF FLUIDS ***'
 10   OPEN (FILEN, FILE = FXNAME, STATUS = 'OLD', ERR=20)
      GOTO 40
 20   OPEN (FILEN, FILE = 'C:\SOURCE\'//FXNAME, STATUS='OLD', ERR=30)
      WRITE (*,*) ' Using reference file from SOURCE directory:',FXNAME
      GOTO 40
 30   OPEN (FILEN, FILE = 'C:\ALLPROPS\'//FXNAME, STATUS='OLD', ERR=35)
      WRITE (*,*) ' Using reference file from ALLPROPS directory'
      GOTO 40
 35   WRITE (*,*) ' ERROR:  Fluid specific reference file not found:',
     &              FXNAME
      WRITE (*,*) ' Re-enter reference file name, including path:  '
      READ (*,*) FXNAME
      GOTO 10

C  ...Set/reset calculation flags to clear calculations for previous fluids.
C  ...This is very important when READREF is called a second time.
 40   NDX = INDX
      NME(NDX) = FXNAME
      DALFLAGS    = '0000000000000000'
      IDGDONEQ(0) = .FALSE.
      IDGDONEQ(1) = .FALSE.
      IDGDONEQ(2) = .FALSE.
      SOLIDLINE(NDX) = .FALSE.
      NDX0        = 0
      DEL0        = 0
      TAU0        = 0
      DHH(NDX)    = 0
      SHH(NDX)    = 0
      HHH(NDX)    = 0
      DLH(NDX)    = 0
      DLL(NDX)    = 0
      SMIN(NDX)   = 0
      HMIN(NDX)   = 0
      DSATL(NDX)  = 0
      DSATV(NDX)  = 0
      PSATL(NDX)  = 0
      PSATV(NDX)  = 0
      SSATL(NDX)  = 0
      SSATV(NDX)  = 0
      HSATL(NDX)  = 0
      HSATV(NDX)  = 0
      TDH1(NDX)   = 0
      PDH1(NDX)   = 0
      SDH1(NDX)   = 0
      HDH1(NDX)   = 0
      TDH2(NDX)   = 0
      HDH2(NDX)   = 0
      SDH2(NDX)   = 0
      TSLD(NDX)   = 0
      PSLDH(NDX)  = 0
      PSLDL(NDX)  = 0
      DSLDL(NDX)  = 0
      DSLDH(NDX)  = 0
      SSLDH(NDX)  = 0
      HSLDH(NDX)  = 0
      CP0FLAGQ(INDX) = 0
      PVFLAGQ(INDX)  = 0
      PMFLAGQ(INDX)  = 0
      DLFLAGQ(INDX)  = 0
      DVFLAGQ(INDX)  = 0

C  ...Check for header.  If not found, print error message and stop.
      READ (FILEN, '(A80)') HEADRQ
      IF (INDEX(HEADRQ, ',v 3.4').EQ.0) THEN
        WRITE (*,*) 'The ASCII reference file for fluid ',FXNAME(1:40)
        WRITE (*,*) 'is in incompatible format.'
        STOP
      ENDIF
C-----------------------------------------------------------------------
C  INITIALIZE VARIABLES IN /FLUID/
C
C    NDX    - Current fluid index.
C    IDGPRP - Ideal gas property flag.
C    NAME   - Fluid specific character identification string.
C    HEADRQ - Holds RCS Header for reference file.
C    TNAMEQ - Long name of fluid, used in titles
C-----------------------------------------------------------------------
      CALL NEXTREC(FILEN, RECORD)
      READ (RECORD, '(A25)') TNAMEQ(NDX)
C-----------------------------------------------------------------------
C  INITIALIZE VARIABLES IN /CONST/
C
C    TC   - Critical point temperature. (K)
C    PC   - Critical point pressure. (MPa)
C    DC   - Critical point density. (mol/dm3)
C    TTRP - Triple point temperature. (K)
C    PTRP - Triple point pressure. (MPa)
C    AMOL - Molecular weight. (g/mol)
C    RGAS - Universal molar gas constant. (kJ/mol-K)
C    AF   - Pitzer's acentric factor.
C    MXWFLG ----- Identifies how saturation properties are calculated:
C                 MXWFLG =  1.0:     Maxwell solution.
C                 0.0 < MXWFLG < 1.0:Maxwell solution up to MXFLG*T/Tc
C                 MXWFLG =  0.0:     Intersection of T and Pv with EOS.
C                 MXWFLG = -1.0:     Intersection of T and Pv with EOS
C                                    to get Dv, Ancillary equation for Dl.
C                 MXWFLG = -2.0:     Ancillary equations.
C    EOS -------- Identifies the equation of state type:
C                 EOS = 1            Helmholtz Equation.
C                 EOS = 2            Martin Hou.
C                 EOS = 3            Wagner methane equation of state.
C                 EOS = 4            Ammonia equation of state.
C-----------------------------------------------------------------------
      CALL NEXTREC(FILEN, RECORD)
      READ (RECORD,*,END=50) DC(NDX), PC(NDX), TC(NDX), TSCL(NDX)
 50   IF (TSCL(NDX).EQ.0) TSCL(NDX) = 68
      CALL NEXTREC(FILEN, RECORD)
      READ (RECORD,*) PTRP(NDX), TTRP(NDX), MXWFLG(NDX), EOS(NDX)
      CALL NEXTREC(FILEN, RECORD)
      READ (RECORD,*) AMOL(NDX), RGAS(NDX), AF(NDX)
C-----------------------------------------------------------------------
C  INITIALIZE VARIABLES IN /LIMITS/
C
C    DLOW, DHIGH - Lower and upper density bounds for the EOS. (mol/dm3)
C    PLOW, PHIGH - Lower and upper pressure bounds for the EOS. (MPa)
C    TLOW, THIGH - Lower and upper temperature bounds for the EOS. (K)
C-----------------------------------------------------------------------
      DLOW(NDX) = 1.D-9
      PLOW(NDX) = 1.D-6
      CALL NEXTREC(FILEN, RECORD)
      READ (RECORD,*) DHIGH(NDX), PHIGH(NDX), THIGH(NDX), TLOW(NDX)
C-----------------------------------------------------------------------
C  INITIALIZE VARIABLES IN /REFPRP/
C
C    P0   - Reference state pressure. (MPa)
C    T0   - Reference state temperature. (K)
C    H0   - Reference state enthalpy divided by RGAS.
C    S0   - Reference state entropy divided by RGAS.
C    SMIX - Dimensionless ideal gas entropy of mixing.
C-----------------------------------------------------------------------
      CALL NEXTREC(FILEN, RECORD)
      READ (RECORD,*) P0(NDX), T0(NDX)
      CALL NEXTREC(FILEN, RECORD)
      READ (RECORD,*) H0(NDX), S0(NDX), SMIX(NDX)

C-----------------------------------------------------------------------
C  INITIALIZE VARIABLES IN /EOS/
C
C    NT            - Number of terms in the EOS.
C    COEF          - Array of coefficients for the EOS.
C    IEX, EXJ, LEX - Arrays of exponents for the terms of the EOS.
C    GAMMA         - Nonlinear factor in the argument of the exponential.
C-----------------------------------------------------------------------
      CALL NEXTREC(FILEN, RECORD)
      READ (RECORD,*) NT(NDX)
      IF (NT(NDX).NE.0) CALL NEXTREC(FILEN, RECORD)
      DO 100 K = 1, NT(NDX)
        READ (FILEN,'(A80)') RECORD
        READ (RECORD,*) COEF(K, NDX), IEX(K, NDX),
     &                  EXJ(K, NDX),  LEX(K, NDX), GAMMA(K, NDX)
 100  CONTINUE

      IF (EOS(NDX).EQ.2) THEN
        DO 200 K=1, 8
          A0(K) = 0
          B0(K) = 0
          C0(K) = 0
 200    CONTINUE
        BMH = 0.D0
        UMH = 0.D0
        CPRIME = 0.D0
        DCONV = 1.D0
        KMH = 0.D0
        DO 210 K=1, NT(NDX)
          IF (IEX(K, NDX).EQ.1 .AND. EXJ(K, NDX).EQ.0) THEN
            BMH = COEF(K, NDX)
          ELSEIF (IEX(K, NDX).EQ.1 .AND. EXJ(K, NDX).EQ.1) THEN
            DCONV = COEF(K, NDX)
          ELSEIF (IEX(K, NDX).EQ.1 .AND. EXJ(K, NDX).EQ.2) THEN
            UMH = COEF(K, NDX)
          ELSEIF (IEX(K, NDX).EQ.1 .AND. EXJ(K, NDX).EQ.3) THEN
            CPRIME = COEF(K, NDX)
          ELSEIF (IEX(K, NDX).EQ.0) THEN
            IF (LEX(K, NDX).EQ.1) THEN
              C0(1) = COEF(K, NDX)
              KMH = GAMMA(K, NDX)
            ELSEIF (EXJ(K, NDX).EQ.1) THEN
              B0(1) = COEF(K, NDX)
            ELSEIF (EXJ(K, NDX).EQ.0) THEN
              A0(1) = COEF(K, NDX)
            ENDIF
          ELSEIF (LEX(K, NDX).EQ.1) THEN
            C0(IEX(K, NDX)) = COEF(K, NDX)
            KMH = GAMMA(K, NDX)
          ELSEIF (EXJ(K, NDX).EQ.1) THEN
            B0(IEX(K, NDX)) = COEF(K, NDX)
          ELSEIF (EXJ(K, NDX).EQ.0) THEN
            A0(IEX(K, NDX)) = COEF(K, NDX)
          ENDIF
 210    CONTINUE
      ENDIF
C---Read cp0 coefficients-----------------------------------------------
 110  READ (FILEN, '(A80)', ERR=999, END=999) RECORD
      IF (RECORD.EQ.'CP0') THEN
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) CP0FLAGQ(NDX)
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) NTCP0Q(NDX)
        DO 120 K = 1, NTCP0Q(NDX)
 120    READ (FILEN,*) PWRCP0Q(K,NDX), CP0COEFQ(K,NDX), CP0COEF2Q(K,NDX)
C---Read vapor pressure coefficients------------------------------------
      ELSEIF (RECORD.EQ.'PV') THEN
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) PVFLAGQ(NDX)
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) NTPVQ(NDX)
        DO 130 K = 1, NTPVQ(NDX)
 130    READ (FILEN,*) PWRPVQ(K, NDX), PVCOEFQ(K, NDX)
C---Read melting pressure coefficients----------------------------------
      ELSEIF (RECORD.EQ.'PM') THEN
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) PMFLAGQ(NDX)
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) NTPMQ(NDX)
        DO 135 K = 1, NTPMQ(NDX)
 135    READ (FILEN,*) PWRPMQ(K, NDX), PMCOEFQ(K, NDX)
C---Read liquid density coefficients------------------------------------
      ELSEIF (RECORD.EQ.'DL') THEN
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) DLFLAGQ(NDX)
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) NTDLQ(NDX)
        DO 140 K = 1, NTDLQ(NDX)
 140    READ (FILEN,*) PWRDLQ(K, NDX), DLCOEFQ(K, NDX)
C---Read vapor density coefficients-------------------------------------
      ELSEIF (RECORD.EQ.'DV') THEN
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) DVFLAGQ(NDX)
        CALL NEXTREC(FILEN, RECORD)
        READ (RECORD,*) NTDVQ(NDX)
        DO 150 K = 1, NTDVQ(NDX)
 150    READ (FILEN,*) PWRDVQ(K, NDX), DVCOEFQ(K, NDX)
C---Special terms for methane-------------------------------------------
      ELSEIF (RECORD.EQ.'SPECIAL TERMS') THEN
        IF (EOS(NDX).EQ.3) THEN
          CALL NEXTREC(FILEN,RECORD)
          READ (RECORD,*) SPMETH
          DO 160 K = 1, SPMETH
 160      READ (FILEN,*) ALPHAQ(K), BETAQ(K), SIGMAQ(K)
        ENDIF
      ENDIF
      GOTO 110

 999  CLOSE (FILEN)
      IF (FLAG.EQ.0 .AND. MXWFLG(NDX).NE.-3) THEN
        IDGTEMP = IDGPRP
        IDGPRP = .FALSE.
        H0(NDX) = H0(NDX) / 1000.D0 / RGAS(NDX) - IDGAS(T0(NDX), 1)
        S0(NDX) = S0(NDX) / 1000.D0 / RGAS(NDX) - IDGAS(T0(NDX), 2)
        CALL SETBOUNDS
        IDGPRP = IDGTEMP
      ENDIF
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WRITEREF(FXNAME, INDX)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C    This routine writes a fluid reference file.  Be careful,
C    it overwrites the old one!
C
C  DESCRIPTION OF ARGUMENTS:
C    FXNAME  - Filespec for the fluid reference file. (Input)
C    INDX    - Integer constant that specifies the storage location
C              of the coefficients. (Input)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      CHARACTER FXNAME*(*)
      INTEGER INDX

C--Functions Called---------------------
      REAL*8 IDGAS

C--Local Variables----------------------
      INTEGER K, FILEN
      REAL*8 H0OUT, S0OUT

      IF (EOS(NDX).EQ.3)
     &  PAUSE '***WRITEREF:  METHANE REQUIRES SPECIAL TERMS***'
      FILEN = 65
      OPEN (UNIT = FILEN, FILE = FXNAME)
      WRITE (FILEN, '(A80)') HEADRQ
      WRITE (FILEN, 1300)
      WRITE (FILEN, 1310)
      WRITE (FILEN, 1300)
      WRITE (FILEN, '(A25)') TNAMEQ(INDX)
      WRITE (FILEN, 1000)
      WRITE (FILEN, 2005) DC(INDX), PC(INDX), TC(INDX), TSCL(INDX)
      WRITE (FILEN, 1010)
      WRITE (FILEN, 2010) PTRP(INDX), TTRP(INDX), MXWFLG(INDX),EOS(INDX)
      WRITE (FILEN, 1020)
      WRITE (FILEN, 2000) AMOL(INDX), RGAS(INDX), AF(INDX)
      WRITE (FILEN, 1040)
      WRITE (FILEN, 2000) DHIGH(INDX),PHIGH(INDX),THIGH(INDX),TLOW(INDX)
      WRITE (FILEN, 1050)
      WRITE (FILEN, 2000) P0(INDX), T0(INDX)
      WRITE (FILEN, 1060)
      H0OUT = H0(NDX)
      S0OUT = S0(NDX)
      IF (DHH(NDX).NE.0) THEN
        H0OUT = (H0(NDX) + IDGAS(T0(NDX), 1))*1000.D0*RGAS(NDX)
        S0OUT = (S0(NDX) + IDGAS(T0(NDX), 2))*1000.D0*RGAS(NDX)
      ENDIF
      WRITE (FILEN, 2000) H0OUT, S0OUT, SMIX(INDX)
      WRITE (FILEN, 1070)
      WRITE (FILEN, '(I10)') NT(INDX)
      WRITE (FILEN, 1080)
      WRITE (FILEN, 1090)
      DO 100 K = 1, NT(INDX)
 100  WRITE (FILEN, 1100) COEF(K, INDX), IEX(K, INDX), EXJ(K, INDX),
     &                    LEX(K,INDX), GAMMA(K, INDX)
C---WRITE CP0 COEFFICIENTS----------------------------------------------
      IF (CP0FLAGQ(NDX).NE.0) THEN
        WRITE (FILEN,1200)
        WRITE (FILEN,'(A3)') 'CP0'
        WRITE (FILEN,1160)
        WRITE (FILEN,*) CP0FLAGQ(NDX)
        WRITE (FILEN,1120)
        WRITE (FILEN,*) NTCP0Q(NDX)
        DO 120 K = 1, NTCP0Q(NDX)
 120    WRITE(FILEN,1110)PWRCP0Q(K,NDX),CP0COEFQ(K,NDX),CP0COEF2Q(K,NDX)
      ENDIF
C---WRITE VAPOR PRESSURE COEFFICIENTS-----------------------------------
      IF (PVFLAGQ(NDX).NE.0) THEN
        WRITE (FILEN,1200)
        WRITE (FILEN,'(A2)') 'PV'
        WRITE (FILEN,1170)
        WRITE (FILEN,*) PVFLAGQ(NDX)
        WRITE (FILEN,1130)
        WRITE (FILEN,*) NTPVQ(NDX)
        DO 140 K = 1, NTPVQ(NDX)
 140    WRITE (FILEN, 1110) PWRPVQ(K, NDX), PVCOEFQ(K, NDX)
      ENDIF
C---WRITE MELTING PRESSURE COEFFICIENTS---------------------------------
      IF (PMFLAGQ(NDX).NE.0) THEN
        WRITE (FILEN,1200)
        WRITE (FILEN,'(A2)') 'PM'
        WRITE (FILEN,1175)
        WRITE (FILEN,*) PMFLAGQ(NDX)
        WRITE (FILEN,1135)
        WRITE (FILEN,*) NTPMQ(NDX)
        DO 150 K = 1, NTPMQ(NDX)
 150    WRITE (FILEN, 1110) PWRPMQ(K, NDX), PMCOEFQ(K, NDX)
      ENDIF
C---WRITE LIQUID DENSITY COEFFICIENTS-----------------------------------
      IF (DLFLAGQ(NDX).NE.0) THEN
        WRITE (FILEN,1200)
        WRITE (FILEN,'(A2)') 'DL'
        WRITE (FILEN,1180)
        WRITE (FILEN,*) DLFLAGQ(NDX)
        WRITE (FILEN,1140)
        WRITE (FILEN,*) NTDLQ(NDX)
        DO 160 K = 1, NTDLQ(NDX)
 160    WRITE (FILEN, 1110) PWRDLQ(K, NDX), DLCOEFQ(K, NDX)
      ENDIF
C---WRITE VAPOR DENSITY COEFFICIENTS------------------------------------
      IF (DVFLAGQ(NDX).NE.0) THEN
        WRITE (FILEN,1200)
        WRITE (FILEN,'(A2)') 'DV'
        WRITE (FILEN,1190)
        WRITE (FILEN,*) DVFLAGQ(NDX)
        WRITE (FILEN,1150)
        WRITE (FILEN,*) NTDVQ(NDX)
        DO 180 K = 1, NTDVQ(NDX)
 180    WRITE (FILEN, 1110) PWRDVQ(K ,NDX), DVCOEFQ(K, NDX)
      ENDIF
      CLOSE (FILEN)
C---FORMAT STATEMENTS---------------------------------------------------
 1000 FORMAT ('#  Dc (mol/dm^3)       Pc (MPa)            Tc (K)',
     &      '              Temp. Scale ')
 1010 FORMAT(/'#  Ptrp (MPa)          Ttrp (K)            Maxwell Flag',
     &      '        EOS Type')
 1020 FORMAT(/'#  Mol. Wt. (g/mol)    Gas Const (kJ/mol-K) Pitzer''s',
     &      ' Acentric Factor')
 1040 FORMAT(/'#  Dhigh (mol/dm^3)    Phigh (MPa)         Thigh (K)',
     &      '           Tlow (K)')
 1050 FORMAT(/'#  P0 (MPa)            T0 (K)')
 1060 FORMAT(/'#  H0 (J/mol)          S0 (J/mol-K)        Smix ',
     &       '(J/mol-K)')
 1070 FORMAT(/'#  Number of Terms')
 1080 FORMAT(/'#  Coefficients         i    j      l     Gamma')
 1090 FORMAT('---------------------------------------------------')
 1100 FORMAT (D20.12, I5, F7.3, I5, F14.8, I5, 2F7.3)
 1110 FORMAT (G20.10,5X,2D20.12)
 1120 FORMAT ('#  Number of Cp0 Terms')
 1130 FORMAT ('#  Number of Vapor Pressure Terms')
 1135 FORMAT ('#  Number of Melting Pressure Terms')
 1140 FORMAT ('#  Number of Liquid Density Terms')
 1150 FORMAT ('#  Number of Vapor Density Terms')
 1160 FORMAT ('#  Cp0 Equation Type')
 1170 FORMAT ('#  Vapor Pressure Equation Type')
 1175 FORMAT ('#  Melting Pressure Equation Type')
 1180 FORMAT ('#  Liquid Density Equation Type')
 1190 FORMAT ('#  Vapor Density Equation Type')
 1200 FORMAT (///,'#-----------------------------------------------')
 1300 FORMAT ('#---------------------------------------------------',
     &        '--------------------')
 1310 FORMAT ('# (C) Copyright 1993  |  ',
     &        'Center for Applied Thermodynamic Studies',/,
     &        '#                     |  ',
     &        'University of Idaho, Moscow, Idaho')
 2000 FORMAT (G18.10, 3G20.10)
 2005 FORMAT (G18.10, 2G20.10, 5X, I2)
 2010 FORMAT (G18.10, 2G20.10, 5X, I1)
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE SETBOUNDS
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    To set various bounds used in table generation.
C
C    Be careful about calling SFNDA rather than S2PEQ on the phase
C    boundaries since the claperyon equation is included in S2PEQ.
C    Calling SFNDA will give the wrong results if D>DHIGH
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Functions Called---------------------
      REAL*8 P2PEQ, S2PEQ, H2PEQ, TOFP2P, TOFD2P
      REAL*8 DOFPT, TOFDP, SFNDA, HFNDA, PFNDA

C--Local Variables----------------------
      REAL*8 DL, DV, PL, PV, TL, TV

C...Check that Dhigh<Dsat when Thigh<Tc
      IF (DHIGH(NDX).LT.DC(NDX) .AND. THIGH(NDX).LT.TC(NDX)) THEN
        CALL SATT(THIGH(NDX), 0, DL, DV, PL, PV)
        IF (DHIGH(NDX).GT.DV) DHIGH(NDX) = DV
        IF (PHIGH(NDX).GT.PV) THEN
          PHIGH(NDX) = PV
        ELSE
          CALL SATP(PHIGH(NDX), 0, DL, DV, TL, TV)
          IF (DHIGH(NDX).GT.DV) DHIGH(NDX) = DV
        ENDIF
      ENDIF

C...Get bounds at Thigh and Phigh
      DHH(NDX) = DOFPT(PHIGH(NDX), THIGH(NDX), 0)
      IF (DHH(NDX).GT.DHIGH(NDX)) THEN
        DHH(NDX) = DHIGH(NDX)
        PHIGH(NDX) = PFNDA(DHIGH(NDX), THIGH(NDX))
      ENDIF
      SHH(NDX) = SFNDA(DHH(NDX), THIGH(NDX))
      HHH(NDX) = HFNDA(DHH(NDX), THIGH(NDX))

C...Find saturation properties at Tlow
      CALL SATT(TLOW(NDX), 0, DSATL(NDX), DSATV(NDX),
     &              PSATL(NDX), PSATV(NDX))
      HSATL(NDX) = HLQ
      SSATL(NDX) = SLQ
      SSATV(NDX) = SFNDA(DSATV(NDX), TLOW(NDX))
      HSATV(NDX) = HFNDA(DSATV(NDX), TLOW(NDX))

C...Get bounds at Plow
      IF (PSATL(NDX).LT.PLOW(NDX)) PLOW(NDX) = PSATL(NDX)
      DLH(NDX) = PLOW(NDX)/RGAS(NDX)/THIGH(NDX)
      DLL(NDX) = PLOW(NDX)/RGAS(NDX)/TLOW(NDX)

C...Check to see if equation has a melting line
      SOLIDLINE(NDX) = .TRUE.
      IF (P2PEQ(TLOW(NDX), 3) .EQ. P2PEQ(THIGH(NDX), 3)) THEN
        SOLIDLINE(NDX) = .FALSE.
        IF (DHIGH(NDX).GT.DSATL(NDX)) DHIGH(NDX) = DSATL(NDX)
      ENDIF

      TDH1(NDX)  = TLOW(NDX)
      PDH1(NDX)  = PSATL(NDX)
      HDH1(NDX)  = HSATL(NDX)
      SDH1(NDX)  = SSATL(NDX)
      TSLD(NDX)  = TLOW(NDX)
      PSLDL(NDX) = PSATL(NDX)
      PSLDH(NDX) = PSATL(NDX)
      DSLDH(NDX) = DSATL(NDX)
      DSLDL(NDX) = DSATL(NDX)
      SSLDH(NDX) = SSATL(NDX)
      HSLDH(NDX) = HSATL(NDX)

C...Find saturation temperature at Dhigh (TDH1)
      IF (DHIGH(NDX).LT.DSATL(NDX)) THEN
        CALL SATD(DHIGH(NDX), 0, DV, PDH1(NDX), TDH1(NDX))
        IF (DHIGH(NDX).GT.DC(NDX)) THEN
          HDH1(NDX) = HLQ
          SDH1(NDX) = SLQ
        ELSE
          HDH1(NDX) = HVQ
          SDH1(NDX) = SVQ
        ENDIF
      ENDIF

C...Find where melting line crosses high pressure line
      IF (SOLIDLINE(NDX)) THEN
        TSLD(NDX)  = TOFP2P(PHIGH(NDX), 3)
        PSLDH(NDX) = PHIGH(NDX)
        PSLDL(NDX) = P2PEQ(TLOW(NDX), 3)
        DSLDH(NDX) = DOFPT(PSLDH(NDX), TSLD(NDX), 0)
        DSLDL(NDX) = DOFPT(PSLDL(NDX), TLOW(NDX), 1)
        IF (DHIGH(NDX).GT.DSLDH(NDX)) DHIGH(NDX) = DSLDH(NDX)
        IF (DHIGH(NDX).LT.DSLDH(NDX)) THEN
          TSLD(NDX)  = TOFD2P(DHIGH(NDX), 3)
          PSLDH(NDX) = P2PEQ(TSLD(NDX), 3)
          DSLDH(NDX) = DHIGH(NDX)
        ENDIF
        SSLDH(NDX) = SFNDA(DSLDH(NDX), TSLD(NDX))
        HSLDH(NDX) = HFNDA(DSLDH(NDX), TSLD(NDX))
      ENDIF

C.....Find Minimum enthalpy and entropy
      IF (DHIGH(NDX).GT.DSLDL(NDX)) THEN
        SMIN(NDX) = S2PEQ(TLOW(NDX), 3)
        HMIN(NDX) = H2PEQ(TLOW(NDX), 3)
        IF (SMIN(NDX).GT.SSATL(NDX)) THEN
          SMIN(NDX) = SSATL(NDX)
          HMIN(NDX) = HSATL(NDX)
        ENDIF
      ELSE
        SMIN(NDX) = SSATL(NDX)
        HMIN(NDX) = HSATL(NDX)
      ENDIF

C...Find Maximum enthalpy and entropy
      HMAX(NDX) = HFNDA(DLH(NDX), THIGH(NDX))
      SMAX(NDX) = SFNDA(DLH(NDX), THIGH(NDX))

C...Find where Dhigh line crosses Phigh line or Thigh line (TDH)
      TDH2(NDX) = TSLD(NDX)
      HDH2(NDX) = HSLDH(NDX)
      SDH2(NDX) = SSLDH(NDX)
      IF (DHIGH(NDX).LE.DSLDH(NDX)) THEN
        TDH2(NDX) = TOFDP(DHIGH(NDX), PHIGH(NDX))
        HDH2(NDX) = HFNDA(DHIGH(NDX), TDH2(NDX))
        SDH2(NDX) = SFNDA(DHIGH(NDX), TDH2(NDX))
      ENDIF
      IF (DHIGH(NDX).LT.DC(NDX) .AND. MXWFLG(NDX).GT.-1) MXWFLG(NDX)=-1

C     WRITE (*,*)    'PHIGH, DHIGH    ', PHIGH(NDX), DHIGH(NDX)
C     WRITE (*,*)    'PSATL           ', PSATL(NDX)
C     WRITE (*,*)
C     WRITE (*,*)    'TSLD            ', TSLD(NDX)
C     WRITE (*,*)    'PSLDH, PSLDL    ', PSLDH(NDX), PSLDL(NDX)
C     WRITE (*,*)    'DSLDH, DSLDL    ', DSLDH(NDX), DSLDL(NDX)
C     WRITE (*,*)    'HSLDH, SSLDH    ', HSLDH(NDX), SSLDH(NDX)
C     WRITE (*,*)
C     WRITE (*,*)    'TDH1, PDH1      ', TDH1(NDX), PDH1(NDX)
C     WRITE (*,*)    'HDH1, SDH1      ', HDH1(NDX), SDH1(NDX)
C     WRITE (*,*)    'TDH2, HDH2      ', TDH2(NDX), HDH2(NDX)
C     WRITE (*,*)    'SDH2            ', SDH2(NDX)
C     WRITE (*,*)
C     WRITE (*,*)    'DHH             ', DHH(NDX)
C     WRITE (*,*)    'HHH, SHH        ', HHH(NDX), SHH(NDX)
C     WRITE (*,*)    'DLH, DLL        ', DLH(NDX), DLL(NDX)
C     WRITE (*,*)    'HMIN, SMIN      ', HMIN(NDX), SMIN(NDX)
C     WRITE (*,*)
C     PAUSE
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DHIGH2(P, T)
      INCLUDE 'commons.for'
      REAL*8 DHIGH2, T, P
      DHIGH2 = DHIGH(NDX)
      IF (T.LT.15) THEN
        IF (TNAMEQ(NDX).EQ.'Helium') THEN
          IF (P.GT.4) THEN
            DHIGH2 = 36*P**0.161683D0 + 2
          ELSE
            DHIGH2 = 47
          ENDIF
        ENDIF
      ENDIF
      END
