C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C--MODULE NAME--
C                FUNDIT.FOR
C
C--DESCRIPTION--
C                Contains the iterative routines used when density and/or
C                temperature is not known.
C
C--CONTENTS-----
C  Subroutines:  CLAP(TSAT, DL, DV, PSAT, HLCLAP, HV, SLCLAP, SV)
C                DTOFHP(H, P, D, T)
C                DTOFPS(P, S, D, T)
C                SATP(PSAT, ICALC, DL, DV, TL, TV)
C                SATT(TSAT, ICALC, DL, DV, PL, PV)
C                SATD(DSAT, ICALC, D2, PSAT, TSAT)
C                SATH(HSAT, D1, D2, P1, P2, T1, T2, IPHASE)
C                SATS(SSAT, D, T, IPHASE)
C
C  Functions:    ACNT(I)
C                DNEWT(P, T, DEST)
C                DOFPT(P, T, IPHASE)
C                DOFHT(H, T)
C                DOFST(S, T)
C                DOFT3(T, IPHASE)
C                HOFPT(P, T)
C                H2PEQ(T, IPHASE)
C                SOFPT(P, T)
C                SOLVE(FUNC, IPHASE, FLAG, TOL, Y, X1, X2, Z)
C                SOLV1(FUNC, IPHASE, TOL, X1, X2, Y)
C                SOLVX(FUNC, TOL, X1, X2, Y, Z)
C                SOLVY(FUNC, TOL, X, Y1, Y2, Z)
C                S2PEQ(T, IPHASE)
C                TOFDH(D, H)
C                TOFDP(D, P)
C                TOFDS(D, S)
C                TOFDU(D, U)
C                TOFDX(D, X, FUNC, FUNCNAME)
C                TOFD2P(DSAT, IPHASE)
C                TOFP2P(P, IPHASE)
C
C=======================================================================
C
      FUNCTION ACNT(I)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C    To calculate the acentric factor for a given fluid.
C
C  DESCRIPTION OF ARGUMENTS:
C    ACNT    Acentric Factor
C    I       When I=0, the vapor pressure is calculated using SATT.
C            When I<>0, the vapor pressure is calculated using P2PEQ.
C    PL,PV   Pressure in MPa
C    DL,DV   Density in mol/dm3
C    T       Temperature in Kelvins
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Local Variables----------------------
      INTEGER I
      REAL*8 T, DL, DV, PL, PV, PR, ACNT, P2PEQ

C-----------------------------------------------------------------------
      T = .7D0 * TC(NDX)
      IF (I.EQ.0) THEN
        CALL SATT(T, 0, DL, DV, PL, PV)
      ELSE
        PL = P2PEQ(T, 1)
      ENDIF
      PR = PL / PC(NDX)
      ACNT = -DLOG(PR) / 2.3025851D0 - 1.D0
      END
C=======================================================================
C
      FUNCTION DNEWT(P, T, DEST)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Iterate the function PFNDA for Density given Pressure and
C    Temperature.  Newton's method is used for the iteration.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    P       Pressure in MPa. (Input)
C    T       Temperature in Kelvins. (Input)
C    DEST    Estimate of the Density at P and T in mol/dm3. (Input)
C    DNEWT   Density at P and T in mol/dm3. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Declare Parameters-------------------
      INTEGER ITMAX
      REAL*8 TOL
      PARAMETER (ITMAX = 30)
      PARAMETER (TOL = 0.5D-9)

C--Arguments----------------------------
      REAL*8 P, T, DEST, DNEWT, DNEWTX

C--Functions Called---------------------
      REAL*8 DPDD1, DPDD2, PFNDA

C--Local Variables----------------------
      INTEGER IC
      REAL*8 DELTA, FCN, FCN1, FCN2

C-----------------------------------------------------------------------
C  ITERATE FOR DENSITY USING NEWTON'S METHOD.
C-----------------------------------------------------------------------
      IF (IDGPRP) THEN
        DNEWT = P / (RGAS(NDX) * T)
        RETURN
      ENDIF
      DNEWTX = DEST
C     WRITE (*,'('' DEST,P,T:'',3F15.8)') DEST,p,t
      DO 10 IC = 1, ITMAX
        FCN = PFNDA(DNEWTX, T) - P
        DNEWT = DNEWTX
        IF (DABS(FCN).LE.TOL) GOTO 120
        IF (ERRNUM.NE.0 .AND. IC.GT.10) GOTO 160
        FCN1 = DPDD1(DNEWTX, T)
        FCN2 = DPDD2(DNEWTX, T)
        DELTA = FCN/(FCN2*FCN/2.D0/FCN1 - FCN1)

C    ...If Slope is large, use first order Newton's method
        IF (DABS(DELTA).GT.15.D0) DELTA = -FCN/FCN1
        DNEWTX = DNEWTX + DELTA

C    ...If 2nd order Newton produces a negative density, use first order
        IF (DNEWTX.LE.0) DNEWTX = DNEWTX - DELTA - FCN/FCN1
C       write (*,'(5f15.6)') dnewtx, fcn, fcn1, fcn2, delta
        IF (DNEWTX.LE.0) GOTO 160
        IF (DNEWTX.GT.DHIGH(NDX)*3) GOTO 160
        DNEWT = DNEWTX
        IF (DABS(DELTA/DNEWTX).LE.TOL) GOTO 120
 10   CONTINUE

 160  ERRNUM = 2
 130  CALL ERRORFNC(' DNEWT')
C     CALL ERROR(' Failed to Converge')
      DNEWT = 0
      RETURN

 120  IF (T.LT.TC(NDX)) THEN
        IF (DEST.LT.DC(NDX) .AND. DNEWT.GT.DC(NDX)) ERRNUM = 1
        IF (DEST.GT.DC(NDX) .AND. DNEWT.LT.DC(NDX)) ERRNUM = 1
        IF (ERRNUM.NE.0) GOTO 130
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DOFPT(P, T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Iterate the function PFNDA for Density given Pressure and
C    Temperature.  Quadratic interpolation is used.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    P       Pressure in MPa. (Input)
C    T       Temperature in Kelvins. (Input)
C    IPHASE  Switch to specify how Density bounds are choosen.
C    DOFPT   Density at P and T in mol/dm3. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 P, T, DOFPT

C--Functions Called---------------------
      REAL*8 D2PEQ, PFNDA, P2PEQ, SOLVX, DNEWT, DHIGH2
      EXTERNAL PFNDA

C--Local Variables----------------------
      REAL*8 D1, D2, TOL

C-----------------------------------------------------------------------
C  SETUP BOUNDS FOR THE ITERATION:
C-----------------------------------------------------------------------
      IF (IDGPRP) THEN
        DOFPT = P / (RGAS(NDX) * T)
        RETURN
      ELSEIF (T.GE.TC(NDX)) THEN
        IF (DHIGH(NDX).LT.DC(NDX)) THEN
          D1 = DLOW(NDX)
          D2 = DHIGH2(P, T)
        ELSEIF (P.LT.PFNDA(DC(NDX), T)) THEN
          D1 = DLOW(NDX)
          D2 = DC(NDX)
        ELSE
          D1 = DC(NDX)
          D2 = DHIGH2(P, T)
        ENDIF
      ELSE
        IF (IPHASE.NE.2 .AND.
     &     (IPHASE.EQ.1 .OR. P-P2PEQ(T, 1).GT.0.5D-9)) THEN
C    .....Bound the liquid region.
          D1 = 0.95D0*D2PEQ(T, 1)
          D2 = DHIGH2(P, T)
          IF (DHIGH(NDX).LT.DC(NDX)) THEN
            ERRNUM = 1
            GOTO 20
          ENDIF
        ELSE
C    .....Bound the vapor region.
          D1 = DLOW(NDX)
          IF (DHIGH(NDX).GT.DC(NDX) .OR. T.LT.TDH1(NDX)) THEN
            D2 = 1.05D0*D2PEQ(T, 2)
          ELSE
            D2 = DHIGH(NDX)
          ENDIF
        ENDIF
      ENDIF
      IF (D2.GT.DHIGH(NDX)) D2 = DHIGH(NDX)
C-----------------------------------------------------------------------
C  SOLVE FOR THE DENSITY.
C-----------------------------------------------------------------------
      TOL = 0.5D-9
      IF (DABS(P).GT.0.5) TOL = 0.5D-9*DABS(P)
C     WRITE (*,'(5F12.6)') P, D1, D2, PFNDA(D1, T), PFNDA(D2, T)
      DOFPT = SOLVX(PFNDA, TOL, D1, D2, T, P)
      IF (ERRNUM.NE.0) DOFPT = DNEWT(P, T, D2)
 20   IF (ERRNUM.NE.0) DOFPT = 0
      CALL ERRORFNC(' DOFPT')
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DOFHT(H, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Iterate the function HFNDA for Density given Temperature and
C    Enthalpy.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T       Temperature in K. (Input)
C    H       Enthalpy in J/mol.  (Input)
C    DOFDH   Density at T and H in Kelvins. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 T, H, DOFHT

C--Functions Called---------------------
      REAL*8 HFNDA, SOLVX, DOFPT
      EXTERNAL HFNDA

C--Local Variables----------------------
      REAL*8 D1, D2, TOL

C-----------------------------------------------------------------------
C  SETUP BOUNDS FOR THE ITERATION:
C-----------------------------------------------------------------------
      D1 = DLOW(NDX)
      D2 = DHIGH(NDX)
      IF (H.GT.HDH2(NDX)) D2 = DOFPT(PHIGH(NDX), T, 0)
      TOL = 0.5D-9
      IF (DABS(H).GT.0.5D0) TOL = 0.5D-9*DABS(H)
      DOFHT = SOLVX(HFNDA, TOL, D1, D2, T, H)
      CALL ERRORFNC(' DOFHT')
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DOFST(S, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Iterate the function SFNDA for Density given Entropy and
C    Temperature.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    S       Entropy in J/mol-K. (Input)
C    T       Temperature in Kelvins. (Input)
C    DOFST   Density at S and T in mol/dm3. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 S, T, DOFST

C--Functions Called---------------------
      REAL*8 SFNDA, SOLVX, DOFPT
      EXTERNAL SFNDA

C--Local Variables----------------------
      REAL*8 D1, D2, TOL, DL, DV, PL, PV

C-----------------------------------------------------------------------
C  SETUP BOUNDS FOR THE ITERATION:
C-----------------------------------------------------------------------
      DOFST = 0
      IF (IDGPRP) THEN
        D1 = DLOW(NDX)
        D2 = DHIGH(NDX)
      ELSEIF (T.GE.TC(NDX)) THEN
        D1 = DLOW(NDX)
        D2 = DHIGH(NDX)
        IF (T.GT.TDH2(NDX)) D2 = DOFPT(PHIGH(NDX), T, 0)
      ELSE
        CALL SATT(T, 0, DL, DV, PL, PV)
        IF (DV.EQ.0 .AND. DHIGH(NDX).LT.DC(NDX)) THEN
          D1 = DLOW(NDX)
          D2 = DHIGH(NDX)
        ELSEIF (S.GT.SVQ) THEN
          D1 = DLOW(NDX)
          D2 = DV
        ELSEIF (S.LT.SLQ) THEN
          D1 = DL
          D2 = DHIGH(NDX)
        ELSE
          RETURN
        ENDIF
      ENDIF
      TOL = 0.5D-9
      IF (DABS(S).GT.0.5D0) TOL = 0.5D-9*DABS(S)
C     WRITE (*,*) S, SFNDA(D1,T), SFNDA(D2,T)
      DOFST = SOLVX(SFNDA, TOL, D1, D2, T, S)
      CALL ERRORFNC(' DOFST')
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DOFT3(T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Returns the density of the freezing line at the tempertature T.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T       Saturation temperature in Kelvins. (Input)
C    IPHASE  Used for consistency among labeling buffer routines.
C    DOFT3   Freezing denisity at T in mol/dm3. (Output)
C-----------------------------------------------------------------------

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 T, DOFT3

C--Functions Called---------------------
      REAL*8 P2PEQ, DOFPT

C-----------------------------------------------------------------------
      DOFT3 = DOFPT(P2PEQ(T, IPHASE), T, 1)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION HOFPT(P, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculates Enthalpy as a function of Pressure and Temperature.
C    At the input P and T, first solve for D.  Then use D and T to
C    find Enthalpy.  D is passed back to the calling program in
C    /PASSD/.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    P        Pressure in MPa. (Input)
C    T        Temperature in Kelvins. (Input)
C    HOFPT    Enthalpy at P and T in J/mol-K. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      REAL*8 HOFPT, P, T

C--Functions Called---------------------
      REAL*8 DOFPT, HFNDA

C-----------------------------------------------------------------------
      DENS = DOFPT(P, T, IPHASE1)
      IF (ERRNUM.NE.0) RETURN
      HOFPT = HFNDA(DENS, T)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION H2PEQ(T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Intersect EOS with D2PEQ to calculate density and then enthalpy.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T       Saturation Temperature in Kelvins. (Input)
C    IPHASE  Switch to select liquid or vapor phase. (Input)
C    H2PEQ   Saturation enthalpy. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 T, H2PEQ, H2PEQX

C--Functions Called---------------------
      REAL*8 HFNDA, D2PEQ, P2PEQ

C--Local Variables----------------------
      REAL*8 DL, DV, PL, PV

C-----------------------------------------------------------------------
      DENS = 0
      PRES = 0
      H2PEQX = 0
      IF (IPHASE.LT.1 .OR. IPHASE.GT.3) THEN
        WRITE (*,*) ' ***  H2PEQ: IPHASE NOT SET TO 1, 2, OR 3 *** '
      ELSEIF (IPHASE.EQ.3) THEN
        IF (.NOT.SOLIDLINE(NDX)) RETURN
        DENS = D2PEQ(T, IPHASE)
        PRES = P2PEQ(T, IPHASE)
        H2PEQX = HFNDA(DENS, T)
      ELSE
        CALL SATT(T, 0, DL, DV, PL, PV)
        IF (IPHASE.EQ.1) THEN
          DENS = DL
          PRES = PL
          H2PEQX = HLQ
        ELSE
          DENS = DV
          PRES = PV
          H2PEQX = HVQ
        ENDIF
      ENDIF
      H2PEQ = H2PEQX
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION SOFPT(P, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculates Entropy as a function of Pressure and Temperature.
C    At the input P and T, first solve for D.  Then use D and T to
C    find Entropy.  D is passed back to the calling program in
C    /PASSD/.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    P        Pressure in MPa. (Input)
C    T        Temperature in Kelvins. (Input)
C    SOFPT    Entropy at P and T in J/mol-K. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      REAL*8 SOFPT, P, T

C--Functions Called---------------------
      REAL*8 DOFPT, SFNDA

C-----------------------------------------------------------------------
      DENS = DOFPT(P, T, IPHASE1)
      IF (ERRNUM.NE.0) RETURN
      SOFPT = SFNDA(DENS, T)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION S2PEQ(T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Intersect EOS with D2PEQ to calculate density and then entropy.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T       Saturation Temperature in Kelvins. (Input)
C    IPHASE  Switch to select liquid or vapor phase. (Input)
C    S2PEQ   Saturation entropy. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 T, S2PEQ, S2PEQX

C--Functions Called---------------------
      REAL*8 SFNDA, D2PEQ

C--Local Variables----------------------
      REAL*8 DL, DV, PL, PV

C-----------------------------------------------------------------------
      DENS = 0
      PRES = 0
      S2PEQX = 0
      IF (IPHASE.LT.1 .OR. IPHASE.GT.3) THEN
        WRITE (*,*) ' ***  S2PEQ: IPHASE NOT SET TO 1, 2, OR 3 *** '
      ELSEIF (IPHASE.EQ.3) THEN
        IF (.NOT.SOLIDLINE(NDX)) RETURN
        DENS = D2PEQ(T, IPHASE)
        S2PEQX = SFNDA(DENS,T)
      ELSE
        CALL SATT(T, 0, DL, DV, PL, PV)
        IF (IPHASE.EQ.1) THEN
          DENS = DL
          PRES = PL
          S2PEQX = SLQ
        ELSE
          DENS = DV
          PRES = PV
          S2PEQX = SVQ
        ENDIF
      ENDIF
      S2PEQ = S2PEQX
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION TOFDH(D, H)
C
C  PURPOSE:
C    Iterate the function HFNDA for Temperature given Density and Enthalpy.
C    See function TOFDX.
C-----------------------------------------------------------------------
      REAL*8 TOFDH, D, H, HFNDA, TOFDX
      EXTERNAL HFNDA
      TOFDH = TOFDX(D, H, HFNDA, ' TOFDH')
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION TOFDP(D, P)
C
C  PURPOSE:
C    Iterate the function PFNDA for Temperature given Density and Pressure.
C    See function TOFDX.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      REAL*8 TOFDP, D, P, PFNDA, TOFDX
      EXTERNAL PFNDA
      IF (IDGPRP) THEN
        TOFDP = P / (RGAS(NDX) * D)
      ELSE
        TOFDP = TOFDX(D, P, PFNDA, ' TOFDP')
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION TOFDS(D, S)
C
C  PURPOSE:
C    Iterate the function SFNDA for Temperature given Density and Entropy.
C    See function TOFDX.
C-----------------------------------------------------------------------
      REAL*8 TOFDS, D, S, SFNDA, TOFDX
      EXTERNAL SFNDA
      TOFDS = TOFDX(D, S, SFNDA, ' TOFDS')
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION TOFDU(D, U)
C
C  PURPOSE:
C    Iterate the function UFNDA for Temperature given Density and Energy.
C    See function TOFDX.
C-----------------------------------------------------------------------
      REAL*8 TOFDU, D, U, UFNDA, TOFDX
      EXTERNAL UFNDA
      TOFDU = TOFDX(D, U, UFNDA, ' TOFDU')
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION TOFDX(D, X, FUNC, FUNCNAME)
C
C  PURPOSE:
C    Iterate the function FUNC for Temperature given Density and X.
C    X can be pressure, energy, enthalpy, or entropy.
C
C  DESCRIPTION OF ARGUMENTS:
C    D       Density in mol/dm3. (Input)
C    X       Other independent variable. (Input)
C    TOFDX   Temperature at D and X in Kelvins. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      CHARACTER*6 FUNCNAME
      REAL*8 D, X, TOFDX, FUNC
      EXTERNAL FUNC

C--Functions Called---------------------
      REAL*8 SOLVY

C--Local Variables----------------------
      REAL*8 T1, T2, TOL, D2, P

C-----------------------------------------------------------------------
C  SETUP BOUNDS FOR THE ITERATION:
C-----------------------------------------------------------------------
      TOFDX = 0
      IF (D.GT.DHH(NDX) .AND. DSLDH(NDX).NE.0) THEN
C    ...Estimate an Upper Bound
        T2 = TSLD(NDX) + (D - DSLDH(NDX))*(THIGH(NDX)-TSLD(NDX))
     &                  /(DHH(NDX)-DSLDH(NDX))
C    ...Increase it slightly for bounding purposes.
        T2 = T2 * 1.1D0
        IF (FUNC(D, T2).LT.X) T2 = THIGH(NDX)
        IF (T2.GT.THIGH(NDX)) T2 = THIGH(NDX)
      ELSE
        T2 = THIGH(NDX)
      ENDIF
      IF (IDGPRP) THEN
        T1 = TLOW(NDX)
      ELSEIF (D.GE.DSATL(NDX)) THEN
        T1 = TLOW(NDX)
      ELSEIF (D.LE.DSATV(NDX)) THEN
        T1 = TLOW(NDX)
      ELSE
        CALL SATD(D, 0, D2, P, T1)
        T1 = T1 - 0.5D-5
        IF (D.GT.DC(NDX)) THEN
          IF (FUNCNAME.EQ.' TOFDH') THEN
            IF (X.LT.HLQ) RETURN
          ELSEIF (FUNCNAME.EQ.' TOFDS') THEN
            IF (X.LT.SLQ) RETURN
          ELSEIF (FUNCNAME.EQ.' TOFDU') THEN
            IF (X.LT.ULQ) RETURN
          ENDIF
        ELSE
          IF (FUNCNAME.EQ.' TOFDH') THEN
            IF (X.LT.HVQ) RETURN
          ELSEIF (FUNCNAME.EQ.' TOFDS') THEN
            IF (X.LT.SVQ) RETURN
          ELSEIF (FUNCNAME.EQ.' TOFDU') THEN
            IF (X.LT.UVQ) RETURN
          ENDIF
        ENDIF
      ENDIF
      TOL = 0.5D-9
      IF (DABS(X).GT.0.5D0) TOL = 0.5D-9*DABS(X)
C     WRITE (*,*) T1, T2
C     WRITE (*,*) X, FUNC(D, T1), FUNC(D, T2)
      TOFDX = SOLVY(FUNC, TOL, D, T1, T2, X)
      IF (ERRNUM.NE.0) TOFDX = 0
      CALL ERRORFNC(FUNCNAME)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION TOFD2P(DSAT, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Iterate the function D2PEQ for Temperature given Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    DSAT     Saturation density in mol/dm3. (Input)
C    IPHASE   Switch to specify bubble line, dewline, or melting line
C               If IPHASE = 1, bubble line
C               If IPHASE = 2, dew line
C               If IPHASE = 3, melting line
C    TOFD2P   Saturation Temperature at PSAT in Kelvins. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 DSAT, TOFD2P

C--Functions Called---------------------
      REAL*8 D2PEQ, SOLV1, TOFP2P
      EXTERNAL D2PEQ

C--Local Variables----------------------
      REAL*8 T1, T2, TOL

C-----------------------------------------------------------------------
C  SETUP BOUNDS FOR THE ITERATION AND CALL THE ITERATION ROUTINE:
C-----------------------------------------------------------------------
      T1 = TTRP(NDX)
      IF (IPHASE.NE.3) T1 = T1 - 1
      T2 = TC(NDX)
      IF (IPHASE.EQ.3) T2 = TOFP2P(PHIGH(NDX), 3)
      TOL = 0.5D-9
      IF (DABS(DSAT).GT.0.5D0) TOL = 0.5D-9*DABS(DSAT)
      TOFD2P = SOLV1(D2PEQ, IPHASE, TOL, T1, T2, DSAT)
      CALL ERRORFNC('TOFD2P')
      IF (ERRNUM.NE.0) TOFD2P = TTRP(NDX)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION TOFP2P(P, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Iterate the function P2PEQ for Temperature given Pressure.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    P        Vapor Pressure in MPa. (Input)
C    IPHASE   Switch to specify bubble line, dewline, or melting line
C               If IPHASE = 1, bubble line
C               If IPHASE = 2, dew line
C               If IPHASE = 3, melting line
C    TOFP2P   Saturation Temperature in Kelvins. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 P, TOFP2P

C--Functions Called---------------------
      REAL*8 P2PEQ, SOLV1
      EXTERNAL P2PEQ

C--Local Variables----------------------
      REAL*8 T1, T2, TOL

C-----------------------------------------------------------------------
C  SETUP BOUNDS FOR THE ITERATION:
C-----------------------------------------------------------------------
      T1 = TTRP(NDX)
      IF (PVFLAGQ(NDX).NE.20 .AND. IPHASE.NE.3) T1 = T1 - 1
      T2 = TC(NDX)
      IF (PSATL(NDX).NE.PSATV(NDX) .AND. IPHASE.EQ.2) THEN
C    ...For mixtures on the vapor side, drop Ttrp a little.
        IF (P.LT.PSATL(NDX) .AND. P.GT.PSATV(NDX)) T1 = T1 - 10.D0
      ENDIF
      IF (IPHASE.EQ.3) THEN
        T2 = THIGH(NDX)
        IF (.NOT.SOLIDLINE(NDX)) THEN
          TOFP2P = TLOW(NDX)
          RETURN
        ENDIF
      ENDIF
      TOL = 0.5D-9
      IF (DABS(P).GT.0.5D0) TOL = 0.5D-9*DABS(P)
C     WRITE (*,*) IPHASE
C     WRITE (*,*) P, P2PEQ(T1, IPHASE), P2PEQ(T2,IPHASE)
      TOFP2P = SOLV1(P2PEQ, IPHASE, TOL, T1, T2, P)
      IF (ERRNUM.NE.0 .AND. P.EQ.PTRP(NDX) .OR. P.EQ.PSATL(NDX)) THEN
        TOFP2P = TTRP(NDX)
        ERRNUM = 0
        RETURN
      ENDIF
      IF (IPHASE.EQ.3 .AND. P.LT.P2PEQ(TTRP(NDX),3)) THEN
        TOFP2P = TTRP(NDX)
        ERRNUM = 0
        RETURN
      ENDIF
      CALL ERRORFNC('TOFP2P')
      IF (ERRNUM.NE.0) TOFP2P = TTRP(NDX)
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE CLAP(TSAT, DL, DV, HV, SV, ULCLAP, HLCLAP, SLCLAP)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C     Use the Claperyon equation to determine the enthalpy and
C     entropy of the liquid saturated surface.
C     *** This statement should only be called from SATT or SATP ***
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 TSAT, DL, DV, HV, SV, ULCLAP, HLCLAP, SLCLAP

C--Functions Called---------------------
      REAL*8 P2PEQ

C--Local Variables----------------------
      REAL*8 PSAT, P1, P2, P3, P4, T1, T2, T3, T4, DELT, DPDTX

C-----------------------------------------------------------------------
      ULCLAP = 0
      HLCLAP = 0
      SLCLAP = 0
      IF (TSAT.GT.TC(NDX)) THEN
        WRITE (*,1001) TSAT, TC(NDX), TNAMEQ(NDX)
 1001    FORMAT('  *** CLAP:  TSAT ',F9.4,' > TC ',F9.4,' for ',
     &          A8,' ***')
         RETURN
      ENDIF

      DELT = 0.01D0
      T1 = TSAT - 2.D0*DELT
      T2 = TSAT - DELT
      T3 = TSAT + DELT
      T4 = TSAT + 2.D0*DELT
      PSAT = P2PEQ(TSAT, 1)
      IF (T1.GE.TLOW(NDX)) THEN
        P1 = P2PEQ(T1, 1)
        P2 = P2PEQ(T2, 1)
      ENDIF
      IF (T4.LE.TC(NDX)) THEN
        P3 = P2PEQ(T3, 1)
        P4 = P2PEQ(T4, 1)
      ENDIF

      IF (T1.LT.TLOW(NDX)) THEN
        DPDTX = (-P4 + 4.D0*P3 - 3.D0*PSAT)/(2.D0*DELT)
      ELSEIF (T4.GT.TC(NDX)) THEN
        DPDTX = (3.D0*PSAT - 4.D0*P2 + P1)/(2.D0*DELT)
      ELSE
        DPDTX = (-P4 + 8.D0*P3 - 8.D0*P2 + P1)/(12.D0*DELT)
      ENDIF
      SLCLAP = SV - 1000.D0*DPDTX*(1.D0/DV - 1.D0/DL)
      HLCLAP = HV - 1000.D0*DPDTX*(1.D0/DV - 1.D0/DL)*TSAT
      ULCLAP = HLCLAP - PSAT/DL*1000.D0
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE DTOFHP(H, P, D, T)
C
C  PURPOSE:
C
C    Iterate the function HOFPT for Temperature given Pressure and Enthalpy.
C    See function DTOFXP
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      REAL*8 D, H, P, T
      REAL*8 HFNDA, HOFPT
      EXTERNAL HFNDA, HOFPT
      CALL DTOFXP(H, P, D, T, HFNDA, HOFPT, 'DTOFHP')
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE DTOFPS(P, S, D, T)
C
C  PURPOSE:
C
C    Iterate the function SOFPT for Temperature given Pressure and Entropy.
C    See function DTOFXP
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      REAL*8 D, S, P, T
      REAL*8 SFNDA, SOFPT
      EXTERNAL SFNDA, SOFPT
      CALL DTOFXP(S, P, D, T, SFNDA, SOFPT, 'DTOFPS')
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE DTOFHS(H, S, D, T)
C
C  PURPOSE:
C
C    Iterate the function SOFPT for Temperature given Pressure and Entropy.
C    See function DTOFXP
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INTEGER I
      REAL*8 D, T, H, S, DOLD
      REAL*8 TOFDH, DOFST, HFNDA
      D = 5.D0
      I = 0
 10     DOLD = D
        I = I + 1
        T = TOFDH(D, H)
        IF (T.EQ.0) GOTO 20
        D = DOFST(S, T)
        IF (I.GT.10) WRITE (*,*) D, T, HFNDA(D,T)
      IF (DABS(D-DOLD).GT.0.5D-9 .AND. I.LT.40) GOTO 10
      IF (DABS(HFNDA(D,T)-H).LT.0.5D-5) RETURN
 20   WRITE (*,*) ' DTOFHS FAILED TO CONVERGE '
      D = 0
      T = 0
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE DTOFXP(X, P, D, T, XFNDA, XOFPT, FUNCNAME)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Iterate the function DOFPT for Temperature given Pressure and
C    Enthalpy or Entropy.  Density is passed to this routine in
C    the /PASSD/ common block from the function HOFPT or SOFPT.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    P       Pressure in MPa. (Input)
C    X       Enthalpy in J/mol or Entropy in J/mol-K.  (Input)
C    D       Density in mol/dm3. (Output)
C    T       Temperature in (K). (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      CHARACTER*6 FUNCNAME
      REAL*8 D, X, P, T

C--Functions Called---------------------
      REAL*8 XFNDA, XOFPT, SOLVY, TOFP2P, TOFDP
      EXTERNAL XFNDA, XOFPT

C--Local Variables----------------------
      REAL*8 T1, T2, TOL, TL, TV, DL, DV

C-----------------------------------------------------------------------
C  SETUP BOUNDS FOR THE ITERATION:
C-----------------------------------------------------------------------
      D = 0
      T = 0
      T1 = TLOW(NDX)
      T2 = THIGH(NDX)
      IF (IDGPRP) THEN
      ELSEIF (P.GT.PC(NDX)) THEN
        IPHASE1 = 0
        IF (SOLIDLINE(NDX)) THEN
          T1 = TOFP2P(P, 3)
        ELSE
          T1 = TOFDP(DHIGH(NDX), P)
        ENDIF
      ELSEIF (P.GT.PSATL(NDX)) THEN
        CALL SATP(P, 0, DL, DV, TL, TV)
        IF (DL.EQ.0) THEN
          IF (FUNCNAME.EQ.'DTOFPS') THEN
            IF (X.LT.SDH2(NDX)) T1 = TDH1(NDX)
            IF (X.GE.SDH2(NDX)) T1 = TDH2(NDX)
          ELSEIF (FUNCNAME.EQ.'DTOFHP') THEN
            IF (X.LT.HDH2(NDX) .AND. X.GT.HDH1(NDX)) THEN
              T1 = TOFDP(DHIGH(NDX), P)
              T2 = TDH2(NDX)
            ELSE
              T1 = TDH2(NDX)
            ENDIF
          ENDIF
        ELSEIF (X.GT.XFNDA(DV, TV)) THEN
          T1 = TV
        ELSEIF (X.LT.XFNDA(DL, TL)) THEN
          T2 = TL
          IF (T1.LT.TDH1(NDX)) T1 = TDH1(NDX)
        ELSE
          RETURN
        ENDIF
      ENDIF
      TOL = 0.5D-9
      IF (DABS(X).GT.0.5D0) TOL = 0.5D-9*DABS(X)
C     WRITE (*,*) X, T1, T2
C     WRITE (*,*) P, XOFPT(P,T1),XOFPT(P,T2)
      T = SOLVY(XOFPT, TOL, P, T1, T2, X)
      D = DENS
      IF (ERRNUM.NE.0) D = 0
      IPHASE1 = 0
      CALL ERRORFNC(FUNCNAME)
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE SATP(PSAT, ICALC, DL1, DV1, TL1, TV1)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Find the Saturation Temperatures and Densities on an isobar.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    PSAT   Saturation Pressure in MPa. (Input)
C    ICALC  Switch to select method of solution. (Input)
C             If ICALC = 1, Return estimates from D2PEQ AND P2PEQ.
C             If ICALC = 2, Intersect EOS with P2PEQ.
C             If ICALC = 3, Return a Maxwell Solution.
C    DL1    Liquid State Density in mol/dm3. (Output)
C    DV1    Vapor State Density in mol/dm3. (Output)
C    TL1    Liquid State Temperature in Kelvins. (Output)
C    TV1    Vapor State Temperature in Kelvins. (Output)
C
C  NOTE:  The variables DL1, DV1, TL1 and TV1 are renamed DL, DV, TL and
C           TV in the code to avoid havoc when a user calls the routine
C           with the same variable for density or pressure, i.e.:
C           CALL SATP(P, 0, DL, DV, T, T)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Declare Parameters-------------------
      INTEGER ITMAX
      REAL*8 TOL
      PARAMETER (ITMAX = 30)
      PARAMETER (TOL = 0.5D-7)

C--Arguments----------------------------
      INTEGER ICALC
      REAL*8 PSAT, DL1, DV1, TL1, TV1

C--Functions Called---------------------
      REAL*8 D2PEQ, DNEWT, GFNDA, SFNDA, TOFP2P, UFNDA, HFNDA, DOFPT

C--Local Variables----------------------
      LOGICAL ERRMSG2
      INTEGER IC
      REAL*8 DELTA, DEST, GL, GV, SL, SV, DL, DV, TL, TV

C-----------------------------------------------------------------------
C  BEGIN:
C-----------------------------------------------------------------------
      ERRMSG2 = ERRMSG
      DL1 = 0
      DV1 = 0
      TL1 = 0
      TV1 = 0
      UVQ = 0
      ULQ = 0
      HVQ = 0
      HLQ = 0
      SVQ = 0
      SLQ = 0
      ERRNUM = 0

C.....Do not use SATP when Psat>intersection of Dhigh and sat line
C  ...on vapor side.
      IF (DHIGH(NDX).LT.DC(NDX) .AND. PSAT.GT.PDH1(NDX)) THEN
        ERRMSG = ERRMSG2
        RETURN
      ENDIF

      IF (PSAT.LT.PSATL(NDX)) THEN
        WRITE (*,*) ' ***   SATP: INPUT PRESSURE < PTRP ***'
        IF (IOUT.NE.0)
     &      WRITE (IOUT,*) ' ***   SATP: INPUT PRESSURE < PTRP ***'
        ERRNUM = 1
      ELSEIF (PSAT.GT.PC(NDX)) THEN
        WRITE (*,*) ' ***   SATP: INPUT PRESSURE > PC ***'
        IF (IOUT.NE.0)
     &      WRITE (IOUT,*) ' ***   SATP: INPUT PRESSURE > PC ***'
        ERRNUM = 1
      ELSE
        TL = TOFP2P(PSAT, 1)
        TV = TOFP2P(PSAT, 2)
        DL = D2PEQ(TL, 1)
        DV = D2PEQ(TV, 2)
        TL1 = TL
        TV1 = TV
        DL1 = DL
        DV1 = DV
        IF (ICALC.EQ.1 .OR. TL.GT.0.9997D0*TC(NDX)) GOTO 500
        IF (MXWFLG(NDX).LE.-2) GOTO 500
        DO 30 IC = 1, ITMAX

C    .....Find vapor density
          DEST = DV
          ERRMSG = .FALSE.
          DV = DNEWT(PSAT, TV, DEST)
          IF (ERRNUM.NE.0) DV = DOFPT(PSAT, TV, 2)
          ERRMSG = .TRUE.
          IF (ERRNUM.NE.0 .AND. PSAT.GT.0.89D0*PC(NDX)) GOTO 510
          DV1 = DV
          IF (MXWFLG(NDX).LE.-1) GOTO 500
          IF (TL.LT.TDH1(NDX)) GOTO 500

C    .....Find liquid density
          DEST = DL
          ERRMSG = .FALSE.
          DL = DNEWT(PSAT, TL, DEST)
          IF (ERRNUM.NE.0) DL = DOFPT(PSAT, TL, 1)
          ERRMSG = .TRUE.
          IF (ERRNUM.NE.0 .AND. PSAT.GT.0.89D0*PC(NDX)) GOTO 510
          DL1 = DL

          IF (TL.GE.MXWFLG(NDX)*TC(NDX) .OR. ICALC.EQ.2) GOTO 500
          GL = GFNDA(DL, TL)
          GV = GFNDA(DV, TV)
          SL = SFNDA(DL, TL)
          SV = SFNDA(DV, TV)
          IF (SL - SV.EQ.0.D0) GOTO 140
          DELTA = (GL - GV)/(SL - SV)
          TL = TL + DELTA
          TV = TV + DELTA
          TL1 = TL
          TV1 = TV
          IF (DABS(DELTA).LE.TOL) GOTO 500
 30     CONTINUE
 140    WRITE (*,*) ' ***   SATP: FAILED TO CONVERGE ***'
        IF (IOUT.NE.0)
     &      WRITE (IOUT,*) ' ***   SATP: FAILED TO CONVERGE ***'
        ERRNUM = 2
      ENDIF
      ERRMSG = ERRMSG2
      RETURN

C  ...Recalculate ancillaries when Maxwell solution bombs.
 510  TL1 = TOFP2P(PSAT, 1)
      TV1 = TOFP2P(PSAT, 2)
      DL1 = D2PEQ(TL, 1)
      DV1 = D2PEQ(TV, 2)

C  ...Calculate the saturation energy, enthalpy, and entropy
 500  UVQ = UFNDA(DV1, TV1)
      HVQ = HFNDA(DV1, TV1)
      SVQ = SFNDA(DV1, TV1)
      IF (DL1.GT.DHIGH(NDX)) THEN
        CALL CLAP(TL1, DL1, DV1, HVQ, SVQ, ULQ, HLQ, SLQ)
      ELSE
        ULQ = UFNDA(DL1, TL1)
        HLQ = HFNDA(DL1, TL1)
        SLQ = SFNDA(DL1, TL1)
      ENDIF
      ERRMSG = ERRMSG2
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE SATT(TSAT, ICALC, DL1, DV1, PL1, PV1)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Find the Saturation Pressures and Densities on an isotherm.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    TSAT   Saturation Temperature in Kelvins. (Input)
C    ICALC  Switch to select method of solution. (Input)
C             If ICALC = 1, Return estimates from D2PEQ AND P2PEQ.
C             If ICALC = 2, Intersect EOS with P2PEQ.
C             If ICALC = 3, Return a Maxwell Solution.
C    DL1    Liquid State Density in mol/dm3. (Output)
C    DV1    Vapor State Density in mol/dm3. (Output)
C    PL1    Liquid State Pressure in MPa. (Output)
C    PV1    Vapor State Pressure in MPa. (Output)
C
C  NOTE:  The variables DL1, DV1, PL1 and PV1 are renamed DL, DV, PL and
C           PV in the code to avoid havoc when a user calls the routine
C           with the same variable for density or pressure, i.e.:
C           CALL SATT(T, 0, DL, DV, P, P)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Declare Parameters-------------------
      INTEGER ITMAX
      REAL*8 TOL
      PARAMETER (ITMAX = 30)
      PARAMETER (TOL = 0.5D-7)

C--Arguments----------------------------
      INTEGER ICALC
      REAL*8 TSAT, DL1, DV1, PL1, PV1

C--Functions Called---------------------
      REAL*8 D2PEQ, DNEWT, GFNDA, P2PEQ, UFNDA, HFNDA, SFNDA, DOFPT


C--Local Variables----------------------
      LOGICAL ERRMSG2
      INTEGER IC
      REAL*8 DELTA, DEST, GL, GV, VL, VV, DL, DV, PL, PV

C-----------------------------------------------------------------------
C  BEGIN:
C-----------------------------------------------------------------------
      ERRMSG2 = ERRMSG
      DV1 = 0
      DL1 = 0
      PV1 = 0
      PL1 = 0
      UVQ = 0
      ULQ = 0
      HVQ = 0
      HLQ = 0
      SVQ = 0
      SLQ = 0
      ERRNUM = 0

C.....Do not use SATT when Tsat>intersection of Dhigh and sat line
C  ...on vapor side.
      IF (DHIGH(NDX).LT.DC(NDX) .AND. TSAT.GT.TDH1(NDX)+1
     &    .AND. TDH1(NDX).NE.0) THEN
        ERRMSG = ERRMSG2
        RETURN
      ENDIF

      IF (TSAT.LT.TTRP(NDX)) THEN
        WRITE (*,*) ' ***   SATT: INPUT TEMPERATURE < TTRP ***'
        IF (IOUT.NE.0)
     &      WRITE (IOUT,*) ' ***   SATT: INPUT TEMPERATURE < TTRP ***'
        ERRNUM = 1
      ELSEIF (TSAT.GT.TC(NDX)) THEN
        WRITE (*,*) ' ***   SATT: INPUT TEMPERATURE > TC ***'
        IF (IOUT.NE.0)
     &      WRITE (IOUT,*) ' ***   SATT: INPUT TEMPERATURE > TC ***'
        ERRNUM = 1
      ELSE
        PL = P2PEQ(TSAT, 1)
        PV = P2PEQ(TSAT, 2)
        DL = D2PEQ(TSAT, 1)
        DV = D2PEQ(TSAT, 2)
        PL1 = PL
        PV1 = PV
        DL1 = DL
        DV1 = DV
        IF (ICALC.EQ.1 .OR. TSAT.GT.0.9997D0*TC(NDX)) GOTO 530
        IF (MXWFLG(NDX).LE.-2) GOTO 530

C  .....Reduce vapor density slightly so that the third root is not
C    ...found for temperatures near critical.
        DV = DV/1.01D0
        DO 40 IC = 1, ITMAX

C    .....Find vapor density
          DEST = DV
          ERRMSG = .FALSE.
          DV = DNEWT(PV, TSAT, DEST)
          IF (ERRNUM.NE.0) DV = DOFPT(PV, TSAT, 2)
          ERRMSG = .TRUE.
          IF (ERRNUM.NE.0 .AND. TSAT.GT.0.97D0*TC(NDX)) GOTO 520
          DV1 = DV
          IF (MXWFLG(NDX).LE.-1) GOTO 530
          IF (TSAT.LT.TDH1(NDX)) GOTO 530

C    .....Find liquid density
          DEST = DL
          ERRMSG = .FALSE.
          DL = DNEWT(PL, TSAT, DEST)
          IF (ERRNUM.NE.0) DL = DOFPT(PL, TSAT, 1)
          ERRMSG = .TRUE.
          IF (ERRNUM.NE.0 .AND. TSAT.GT.0.97D0*TC(NDX)) GOTO 520
          DL1 = DL

          IF (TSAT.GE.MXWFLG(NDX)*TC(NDX) .OR. ICALC.EQ.2) GOTO 530
          IF (DL.EQ.0 .OR. DV.EQ.0) THEN
            WRITE (*,*) ' ***  ERROR IN SATT: DENSITY IS ZERO *** '
            IF (IOUT.NE.0)
     &        WRITE (IOUT,*) ' ***  ERROR IN SATT: DENSITY IS ZERO *** '
            ERRMSG = ERRMSG2
            RETURN
          ENDIF
          VL = 1000.D0/DL
          VV = 1000.D0/DV
          GL = GFNDA(DL, TSAT)
          GV = GFNDA(DV, TSAT)
          IF (VL - VV.EQ.0.D0) GOTO 150
          DELTA = (GL - GV)/(VL - VV)
          PL = PL - DELTA
          PV = PV - DELTA
          PL1 = PL
          PV1 = PV
          IF (DABS(DELTA).LE.TOL) GOTO 530
 40     CONTINUE
 150    WRITE (*,*) ' ***   SATT: FAILED TO CONVERGE ***'
        IF (IOUT.NE.0)
     &      WRITE (IOUT,*) ' ***   SATT: FAILED TO CONVERGE ***'
        ERRNUM = 2
      ENDIF
      ERRMSG = ERRMSG2
      RETURN

C  ...Recalculate ancillaries when Maxwell solution bombs.
 520  PL1 = P2PEQ(TSAT, 1)
      PV1 = P2PEQ(TSAT, 2)
      DL1 = D2PEQ(TSAT, 1)
      DV1 = D2PEQ(TSAT, 2)

C  ...Calculate the saturation energy, enthalpy, and entropy
 530  UVQ = UFNDA(DV1, TSAT)
      HVQ = HFNDA(DV1, TSAT)
      SVQ = SFNDA(DV1, TSAT)
      IF (DL1-DHIGH(NDX).GT.0.5D-9 .OR.
     &    (TSAT.LT.TDH1(NDX) .AND. DL1.GT.DC(NDX))) THEN
        CALL CLAP(TSAT, DL1, DV1, HVQ, SVQ, ULQ, HLQ, SLQ)
      ELSE
        ULQ = UFNDA(DL1, TSAT)
        HLQ = HFNDA(DL1, TSAT)
        SLQ = SFNDA(DL1, TSAT)
      ENDIF
      ERRMSG = ERRMSG2
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE SATD(DSAT, ICALC, D2, PSAT, TSAT)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Find the Equilibrium Temperature, Pressure, and Coexistence
C    Densities that satisfy the Maxwell Equal Area Principle
C    given one of the Coexistence Densities.
C
C
C  DESCRIPTION OF ARGUMENTS:
C
C    DSAT    Input Density in mol/dm3. (Input)
C    ICALC   Switch to select method of solution. (Input)
C              If ICALC = 1, Return estimates from D2PEQ AND P2PEQ.
C              If ICALC = 2, Intersect EOS with P2PEQ.
C              If ICALC = 3, Return a Maxwell Solution.
C    D2      Unknown Equilibrium Density in mol/dm3. (Output)
C            Zero is returned for mixtures (such as air).
C    PSAT    Equilibrium Pressure in MPa. (Output)
C    TSAT    Equilibrium Temperature in Kelvins. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Declare Parameters-------------------
      INTEGER ITMAX
      REAL*8 TOL
      PARAMETER (ITMAX = 50)
      PARAMETER (TOL = 1.D-6)

C--Arguments----------------------------
      INTEGER ICALC
      REAL*8 DSAT, D2, PSAT, TSAT

C--Functions Called---------------------
      REAL*8 D2PEQ, DNEWT, GFNDA, P2PEQ, PFNDA, SFNDA, TOFD2P
      REAL*8 UFNDA, HFNDA, SOLVE, DOFPT

C--Local Variables----------------------
      LOGICAL ERRMSG2
      INTEGER I, IC, IPHASE
      REAL*8 DELTA, DEST, FACTOR, G1, G2, S1, S2, DL, DV
      REAL*8 DELTAOLD, TOLD
      REAL*8 PSAT1, PSAT2, PL, PV, SGNX, TT1, TT2

C-----------------------------------------------------------------------
C  INPUT DENSITY IS OUT OF RANGE.
C-----------------------------------------------------------------------
      ERRMSG2 = ERRMSG
      ERRNUM = 0
      D2 = 0
      TSAT = 0
      PSAT = 0
      UVQ = 0
      ULQ = 0
      HVQ = 0
      HLQ = 0
      SVQ = 0
      SLQ = 0
      DELTA = 0
      IF (DSAT.LT.DSATV(NDX)) THEN
        WRITE (*,*) ' ***   SATD: D < DTRP VAPOR ***'
        IF (IOUT.NE.0)
     &      WRITE (IOUT,*) ' ***   SATD: D < DTRP VAPOR ***'
        ERRNUM = 1
      ELSEIF (DSAT.GT.DSATL(NDX)) THEN
        WRITE (*,*) ' ***   SATD: INPUT DENSITY > DTRP LIQUID ***'
        IF (IOUT.NE.0)
     &     WRITE (IOUT,*) ' ***   SATD: INPUT DENSITY > DTRP LIQUID ***'
        ERRNUM = 1
C-----------------------------------------------------------------------
      ELSE
        IPHASE = 1
        IF (DSAT.LT.DC(NDX)) IPHASE = 2
        IF(ICALC.EQ.2 .OR.
     &    (TSAT.GE.MXWFLG(NDX)*TC(NDX) .AND. MXWFLG(NDX).GE.0)) GOTO 400
        IF (MXWFLG(NDX).LE.0) GOTO 400
        IF (TSAT.GT.0.9997D0*TC(NDX) .OR. ICALC.EQ.1) GOTO 400

C-----------------------------------------------------------------------
C  SOLVE THE MAXWELL CRITERION.
C-----------------------------------------------------------------------
        TSAT = TOFD2P(DSAT, IPHASE)
        PSAT = P2PEQ (TSAT, IPHASE)
        PSAT1 = PSAT
        D2   = D2PEQ (TSAT, 3 - IPHASE)
C       PSAT = PFNDA(DSAT, TSAT)
        DO 50 IC = 1, ITMAX
          DEST = D2
          I = 0
          PSAT2 = 1000000
          SGNX = 1
 110      I = I + 1
          IF (I.GT.15) GOTO 400
          ERRMSG = .FALSE.
          IF (TSAT.LE.0.98D0*TC(NDX)) THEN
            D2 = DNEWT(PSAT, TSAT, DEST)
          ELSE
            D2 = DOFPT(PSAT, TSAT, 3 - IPHASE)
          ENDIF
          ERRMSG = .TRUE.
          IF (ERRNUM.NE.0) THEN
C           WRITE (*,'('' BOMB:t,p,d'',3f15.8)') TSAT/TC(NDX),PSAT,DSAT
            IF (PSAT1.LT.0.01D0) GOTO 400
            IF (TSAT.LE.0.9D0*TC(NDX)) THEN
              IF (DABS(PSAT2).LT.DABS(PSAT)) SGNX = -SGNX
              TSAT = TSAT + SGNX*0.05D0
            ELSE
              TSAT = TSAT - 0.005D0
            ENDIF
            PSAT2 = PSAT
            PSAT = PFNDA(DSAT, TSAT)
            GOTO 110
          ENDIF
          IF (TSAT.GE.MXWFLG(NDX)*TC(NDX)) GOTO 540
          IF (TSAT.GT.0.9997D0*TC(NDX) .OR. ICALC.EQ.1) GOTO 400
          G1 = GFNDA(DSAT, TSAT)
          S1 = SFNDA(DSAT, TSAT)
          G2 = GFNDA(D2, TSAT)
          S2 = SFNDA(D2, TSAT)
          DELTAOLD = DELTA
          DELTA = (G1 - G2)/(S1 - S2)
C         WRITE (*,1000) TSAT/TC(NDX),PSAT,D2,DELTA
 1000     FORMAT (' T,P,D2,DELTA:',3F12.5,F20.12)
          IF (DELTAOLD.EQ.0) THEN
            FACTOR = -0.001D0
            IF (IPHASE.EQ.2) FACTOR = 1.D0 + 2.5D0*DSAT/DC(NDX)
          ELSE
            FACTOR = -(TOLD - TSAT)/(DELTAOLD - DELTA)
          ENDIF
          TOLD = TSAT
          TSAT = TSAT + FACTOR*DELTA
          PSAT = PFNDA(DSAT, TSAT)
          IF (DABS(DELTA).LE.TOL) GOTO 540
 50     CONTINUE
        WRITE (*,*) ' ***   SATD: FAILED TO CONVERGE ***'
        IF (IOUT.NE.0)
     &      WRITE (IOUT,*) ' ***   SATD: FAILED TO CONVERGE ***'
        ERRNUM = 2
      ENDIF
      ERRMSG = ERRMSG2
      RETURN

C-----------------------------------------------------------------------
C  ...Iterate SATT to get temperature when ICALC = 2 or MXWFLG = 0
C-----------------------------------------------------------------------
 400  TT1 = TTRP(NDX)
      TT2 = TC(NDX)
      IF (TDH1(NDX).NE.0) THEN
        IF (DHIGH(NDX).LT.DC(NDX)) THEN
          TT2 = TDH1(NDX)
        ELSEIF (DSAT.GT.DC(NDX)) THEN
          IF (DSAT.LE.DHIGH(NDX)) TT1 = TDH1(NDX)
          IF (DSAT.GT.DHIGH(NDX)) TT2 = TDH1(NDX)
        ENDIF
      ENDIF
      TSAT = SOLVE(0.D0, IPHASE, 3, 0.5D-6, 0.D0, TT1, TT2, DSAT)
      CALL SATT(TSAT, 0, DL, DV, PL, PV)
      IF (IPHASE.EQ.1) THEN
        PSAT = PL
        D2 = DV
      ELSE
        PSAT = PV
        D2 = DL
      ENDIF

C  ...Calculate the saturation energy, enthalpy, and entropy
 540  IF (PSATL(NDX).NE.PSATV(NDX)) D2 = 0
      IF (IPHASE.EQ.1) THEN
        DL = DSAT
        DV = D2
      ELSE
        DL = D2
        DV = DSAT
      ENDIF
      IF (DV.NE.0) THEN
        UVQ = UFNDA(DV, TSAT)
        HVQ = HFNDA(DV, TSAT)
        SVQ = SFNDA(DV, TSAT)
      ENDIF
      IF (DL.NE.0) THEN
        IF (DL.GT.DHIGH(NDX)) THEN
          CALL CLAP(TSAT, DL, DV, HVQ, SVQ, ULQ, HLQ, SLQ)
        ELSE
          ULQ = UFNDA(DL, TSAT)
          HLQ = HFNDA(DL, TSAT)
          SLQ = SFNDA(DL, TSAT)
        ENDIF
      ENDIF
      ERRMSG = ERRMSG2
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE SATH(HSAT, D1, D2, P1, P2, T1, T2, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Find the Saturation Temperature and Densities on an Isenthalp.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    HSAT   Saturation Enthalpy in J/Mol. (Input)
C    D      Density in mol/dm3. (Output)
C    P      Pressure in MPa. (Output)
C    T      Temperature in K. (Output)
C    IPHASE Switch to specify how bounds are chosen:
C             Must be set to 3 for melting line, otherwise
C             SATH will set IPHASE to 1 or 2 if enthalpy passes
C             through dome.
C
C           Subscripts 1 and 2 refer to the points on
C           the dome where the constant enthalpy line passes
C           through the dome.  Subscript 1 refers to the higher
C           pressure side and subscript 2 refers to the lower
C           pressure side (when present.)
C
C  ***  Do NOT send IPHASE as a number.  The number's value may get changed ***
C=======================================================================
C  VARIABLES IN /ENTHALPY/
C    HC    - Critical point enthalpy.
C    HTAN  - Enthalpy where the isenthalp is tangent to the saturated
C            vapor line
C    TTAN  - Temperature at HTAN.
C    TTRPH - Used with NAMEH to determine if the fluid has changed since
C            the last call to subroutine SATH.  If it has not changed,
C            then HC, HTAN, and TTAN do not need to be recalculated.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

      CHARACTER*8 NAMEH
      COMMON /ENTHALPC/ NAMEH

      REAL*8 HC, HTAN, TTAN, TTRPH
      COMMON /ENTHALPY/ HC, HTAN, TTAN, TTRPH

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 HSAT, T1, T2, D1, D2, P1, P2

C--Functions Called---------------------
      REAL*8 H2PEQ, SOLV1
      EXTERNAL H2PEQ

C--Local Variables----------------------
      REAL*8 T, TT1, TT2, TOL, TINC, SGNX, H1, DL, DV, PL, PV

C-----------------------------------------------------------------------
C  SET UP BOUNDS:
C-----------------------------------------------------------------------
      IF (TNAMEQ(NDX)(1:8).NE.NAMEH .OR. TTRP(NDX).NE.TTRPH) THEN
        WRITE (*,*) ' Working ...'
        T = TC(NDX)
        IF (TC(NDX).GT.THIGH(NDX)) T = THIGH(NDX)
        IF (DHIGH(NDX).LT.DC(NDX) .AND. T.GT.TDH1(NDX)) T = TDH1(NDX)
        HC = H2PEQ(T, 2)
        HTAN = HC
        TINC = (T - TLOW(NDX))/10.D0
        TTAN = T - TINC
        SGNX = 1.D0
 100    H1 = HTAN
        HTAN = H2PEQ(TTAN, 2)
        IF (H1.GT.HTAN) THEN
          SGNX = -SGNX
          TINC = TINC/2.D0
        ENDIF
        TTAN = TTAN - SGNX*TINC
        IF (TTAN.GT.T) THEN
          TINC = TINC/2.D0
          TTAN = T - TINC
          SGNX = 1.D0
          GOTO 100
        ELSEIF (TINC.GT.0.5D-2) THEN
          GOTO 100
        ENDIF
      ENDIF
      NAMEH = TNAMEQ(NDX)
      TTRPH = TTRP(NDX)

      T1 = 0
      T2 = 0
      D1 = 0
      D2 = 0
      P1 = 0
      P2 = 0
      ERRNUM = 0
      IF ((HSAT.GT.HTAN .OR. HSAT.LT.HSATL(NDX)) .AND. IPHASE.NE.3) THEN
        IPHASE = 0
        RETURN
      ENDIF

      IF (IPHASE.NE.3) THEN
        IF (TC(NDX).GT.THIGH(NDX) .OR. DC(NDX).GT.DHIGH(NDX)) THEN
          IF (TC(NDX).GT.THIGH(NDX)) T1 = THIGH(NDX)
          IF (DHIGH(NDX).LT.DC(NDX)) T1 = TDH1(NDX)
          CALL SATT(T1, 0, DL, DV, PL, PV)
          IF (HSAT.GT.HLQ .AND. HSAT.LT.HVQ) THEN
C           Q = (HSAT - HL)/(HV - HL)
C           D1 = 1.D0/((1.D0 - Q)/DL + Q/DV)
            D1 = DV
            P1 = PV
            IPHASE = 2
            GOTO 200
          ENDIF
          TT2 = THIGH(NDX)
        ENDIF
      ENDIF

      TOL = 0.5D-6
      TT1 = TLOW(NDX)
      TT2 = TC(NDX)
      IF (TT2.GT.THIGH(NDX)) TT2 = THIGH(NDX)
      IF (DHIGH(NDX).LT.DC(NDX) .AND. TT2.GT.TDH1(NDX)) TT2 = TDH1(NDX)
      IF (IPHASE.EQ.3) THEN
        TT2 = TSLD(NDX)
      ELSEIF (HSAT.LT.HC) THEN
        IPHASE = 1
      ELSE
        TT1 = TTAN
        IPHASE = 2
      ENDIF
C     write (*,*) hsat,h2peq(tt1,iphase),h2peq(tt2,iphase)
      T1 = SOLV1(H2PEQ, IPHASE, TOL, TT1, TT2, HSAT)
      D1 = DENS
      P1 = PRES
      CALL ERRORFNC('  SATH')
      IF (ERRNUM.NE.0) RETURN
      IF (IPHASE.EQ.3) RETURN

 200  IF (HSAT.GT.HSATV(NDX)) THEN
        TT1 = TLOW(NDX)
        TT2 = TTAN
        T2 = SOLV1(H2PEQ, 2, TOL, TT1, TT2, HSAT)
        D2 = DENS
        P2 = PRES
        CALL ERRORFNC('  SATH')
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE SATS(SSAT, D, T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Find the Saturation Temperature and Density on an Isentrope.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    SSAT   Saturation Entropy in J/Mol-K. (Input)
C    D      Density in mol/dm3. (Output)
C    T      Temperature in K. (Output)
C    IPHASE Switch to specify how bounds are chosen:
C             Must be set to 3 for melting line, otherwise
C             SATS will set IPHASE to 1 to 2 if entropy passes
C             through dome.
C  ***  Do NOT send IPHASE as a number.  The number's value may get changed ***
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 SSAT, T, D

C--Functions Called---------------------
      REAL*8 S2PEQ, SOLV1, SFNDA
      EXTERNAL S2PEQ

C--Local Variables----------------------
      REAL*8 TT1, TT2, TOL, DL, DV, PL, PV, T1

C-----------------------------------------------------------------------
C  SET UP BOUNDS:
C-----------------------------------------------------------------------
      ERRNUM = 0
      IF ((SSAT.GT.SSATV(NDX) .OR. SSAT.LT.SSATL(NDX))
     &    .AND. IPHASE.NE.3) THEN
        IPHASE = 0
        RETURN
      ENDIF
      TOL = 0.5D-6
      TT1 = TLOW(NDX)
      TT2 = TC(NDX)
      IF (IPHASE.NE.3) THEN
        IF (TC(NDX).GT.THIGH(NDX) .OR. DHIGH(NDX).LT.DC(NDX)) THEN
          IF (TC(NDX).GT.THIGH(NDX)) T1 = THIGH(NDX)
          IF (DHIGH(NDX).LT.DC(NDX)) T1 = TDH1(NDX)
          CALL SATT(T1, 0, DL, DV, PL, PV)
          IF (SSAT.GT.SLQ .AND. SSAT.LT.SVQ) THEN
            T = T1
            D = DV
            RETURN
          ENDIF
        TT2 = T1
        ENDIF
      ENDIF
      IF (IPHASE.EQ.3) THEN
        TT2 = TSLD(NDX)
      ELSEIF (SSAT.LT.SFNDA(DC(NDX),TC(NDX))) THEN
        IPHASE = 1
      ELSE
        IPHASE = 2
      ENDIF
      T = SOLV1(S2PEQ, IPHASE, TOL, TT1, TT2, SSAT)
      IF (ERRNUM.NE.0) DENS = 0
      D = DENS
      CALL ERRORFNC('  SATS')
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE ERRORFNC(FUNCNAME)
      INCLUDE 'commons.for'
      CHARACTER*6 FUNCNAME
      IF (ERRMSG) THEN
        IF (ERRNUM.EQ.1) THEN
          WRITE (*,1000) FUNCNAME
          IF (IOUT.NE.0) WRITE (IOUT,1000) FUNCNAME
        ELSEIF (ERRNUM.EQ.2) THEN
          WRITE (*,1010) FUNCNAME
          IF (IOUT.NE.0) WRITE (IOUT,1010) FUNCNAME
        ENDIF
      ENDIF
 1000 FORMAT ('  *** ',A6,': THE ROOT IS NOT BOUNDED ***')
 1010 FORMAT ('  *** ',A6,': FAILED TO CONVERGE ***')
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION SOLVE(FUNC, IPHASE, FLAG, TOL, Y, XX1, XX2, Z)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C    Find the root, X, of a function of one independent variable,
C    Z = FUNC(X) given values for Z.  This routine will
C    search X in the interval between the variables X1 and X2.
C
C  DESCRIPTION OF ARGUMENTS:
C    FUNC     A Function of the general form, Z = FUNC(X).      (Input)
C    IPHASE   Switch to select liquid, vapor, or melting line.  (Input)
C    TOL      Converge criteria: ABS((FUNC(X) - Z)/Z) < TOL.    (Input)
C    Y        The value of the known independent variable.      (Input)
C    XX1      1st boundary value for the search interval.       (Input)
C    XX2      2nd boundary value for the search interval.       (Input)
C    Z        The value of FUNC at the point of interest.       (Input)
C    SOLVE    The value of X after iteration.                   (Output)
C    FLAG     Tells which variables to use and their order
C               when calling the function:                      (Input)
C               0 --->  Y = FUNC(X, IPHASE)
C               1 --->  Y = FUNC(Y, X)
C               2 --->  Y = FUNC(X, Y)
C  NOTE:
C    X1 and X2 are assigned from XX1 and XX2 because X1 and X2 are
C    changed as the subroutines works.  Thus, if TTRP and TC are sent
C    as the limits, this function will not change their values.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Declare Parameters-------------------
      INTEGER ITMAX
      PARAMETER (ITMAX = 50)

C--Arguments----------------------------
      INTEGER IPHASE, FLAG
      REAL*8 FUNC, TOL, SOLVE, SOLVEV, X1, X2, Y, Z, XX1, XX2
      EXTERNAL FUNC

C--Local Variables----------------------
      INTEGER IC
      REAL*8 F, F1, F2, F3, X3, DL, DV, P

C-----------------------------------------------------------------------
C  CHECK THE BOUNDARIES:
C-----------------------------------------------------------------------
      X1 = XX1
      X2 = XX2
      ERRNUM = 0
      SOLVEV = 0
      SOLVE  = 0

      IF (FLAG.EQ.0) THEN
        F1 = FUNC(X1, IPHASE) - Z
      ELSEIF (FLAG.EQ.1) THEN
        F1 = FUNC(X1, Y) - Z
      ELSEIF (FLAG.EQ.2) THEN
        F1 = FUNC(Y, X1) - Z
      ELSEIF (FLAG.EQ.3) THEN
        CALL SATT(X1, 0, DL, DV, P, P)
        IF (IPHASE.EQ.1) F1 = DL - Z
        IF (IPHASE.EQ.2) F1 = DV - Z
      ENDIF
      IF (ERRNUM.NE.0) RETURN
      IF (DABS(F1).LE.TOL) THEN
        SOLVE = X1
        RETURN
      ENDIF

      IF (FLAG.EQ.0) THEN
        F2 = FUNC(X2, IPHASE) - Z
      ELSEIF (FLAG.EQ.1) THEN
        F2 = FUNC(X2, Y) - Z
      ELSEIF (FLAG.EQ.2) THEN
        F2 = FUNC(Y, X2) - Z
      ELSEIF (FLAG.EQ.3) THEN
        CALL SATT(X2, 0, DL, DV, P, P)
        IF (IPHASE.EQ.1) F2 = DL - Z
        IF (IPHASE.EQ.2) F2 = DV - Z
      ENDIF
      IF (ERRNUM.NE.0) RETURN
      IF (DABS(F2).LE.TOL) THEN
        SOLVE = X2
        RETURN
      ENDIF
      IF (F1*F2.GT.0) THEN
        ERRNUM = 1
        RETURN
      ENDIF
C-----------------------------------------------------------------------
C  BEGIN ITERATING
C-----------------------------------------------------------------------
      DO 60 IC = 1, ITMAX
C    ...Use False Position to get point 3.
        X3 = X1 - F1*(X2 - X1)/(F2 - F1)
        IF (FLAG.EQ.0) THEN
          F3 = FUNC(X3, IPHASE) - Z
        ELSEIF (FLAG.EQ.1) THEN
          F3 = FUNC(X3, Y) - Z
        ELSEIF (FLAG.EQ.2) THEN
          F3 = FUNC(Y, X3) - Z
        ELSEIF (FLAG.EQ.3) THEN
          CALL SATT(X3, 0, DL, DV, P, P)
          IF (IPHASE.EQ.1) F3 = DL - Z
          IF (IPHASE.EQ.2) F3 = DV - Z
        ENDIF
        IF (F1.EQ.F2 .OR. F2.EQ.F3 .OR. F1.EQ.F3) THEN
          IF (F3.GT.0.5D-5)
     &       WRITE (*,*) 'Solve: Function values equal, final root:', F3
          SOLVE = X3
          RETURN
        ELSEIF (MOD(IC,6).EQ.0) THEN
C         write (*,*) 'Solve: Using Bisection Method'
          IF (F1*F3.GT.0) THEN
            SOLVEV = (X2 + X3)/2.D0
          ELSE
            SOLVEV = (X1 + X3)/2.D0
          ENDIF
        ELSE

C      ...Use points 1, 2, and 3 to estimate the root using Chamber's
C      ...method (quadratic solution).
          SOLVEV = X1*F2*F3/((F1 - F2)*(F1 - F3))
     &          + X2*F1*F3/((F2 - F1)*(F2 - F3))
     &          + X3*F1*F2/((F3 - F1)*(F3 - F2))
          IF ((SOLVEV - X1)*(SOLVEV - X2).GE.0) THEN
C           IF (BISECT.EQ.0) THEN
C           WRITE (*,*)'Quadratic solution not bounded, using bisection'
C             BISECT = 1
              SOLVEV = (X1 + X2)/2.D0
C           ELSE
C             WRITE (*,*)'Bisection didn''t work, assuming steep slope'
C             IF (DABS(X1-X3).GT.DABS(X2-X3)) THEN
C               SOLVEV = X1 + 0.05D0*(X2 - X1)
C             ELSE
C               SOLVEV = X2 + 0.05D0*(X1 - X2)
C             ENDIF
C           ENDIF
c           SOLVEV=SOLVE2(FUNC, IPHASE, FLAG, Y, X1, X2, F3, F1, F2, Z)
C           WRITE (*,*) 'SOLVE2: ',SOLVEV
          ENDIF
        ENDIF
        IF (FLAG.EQ.0) THEN
          F = FUNC(SOLVEV, IPHASE) - Z
        ELSEIF (FLAG.EQ.1) THEN
          F = FUNC(SOLVEV, Y) - Z
        ELSEIF (FLAG.EQ.2) THEN
          F = FUNC(Y, SOLVEV) - Z
        ELSEIF (FLAG.EQ.3) THEN
          CALL SATT(SOLVEV, 0, DL, DV, P, P)
          IF (IPHASE.EQ.1) F = DL - Z
          IF (IPHASE.EQ.2) F = DV - Z
        ENDIF
C       write (*,1000) x1, x2, x3, solveV
C       write (*,1000) F1+Z, F2+Z, F3+Z, F+Z
C       write (*,*)
        SOLVE = SOLVEV
        IF (DABS(F).LE.TOL) RETURN

C    ...Discard quadratic solution if false position root is closer.
        IF (DABS(F3).LT.DABS(F) .AND. F*F3.GT.0) THEN
          IF (F3*F1.GT.0) THEN
            X1 = X3
            F1 = F3
          ELSE
            X2 = X3
            F2 = F3
          ENDIF
C         write (*,*) 'Solve: Discarding quadratic solution'
        ELSE

C      ...Swap in new value from quadratic solution
          IF (F*F3.LT.0) THEN
            X1 = SOLVEV
            F1 = F
            X2 = X3
            F2 = F3
          ELSEIF (F3*F1.GT.0) THEN
            X1 = SOLVEV
            F1 = F
           ELSE
            X2 = SOLVEV
            F2 = F
          ENDIF
        ENDIF
 60   CONTINUE
      SOLVE = 0
      ERRNUM = 2
 1000 format (1x, 4g19.12)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION SOLVE2(FUNC, IPHASE, FLAG, Y, X1, X2, F3, F1, F2, Z)
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      INTEGER IPHASE, FLAG
      REAL*8 FUNC, SOLVE2, X, F, X1, X2, F3, F1, F2, Y, Z
      EXTERNAL FUNC

C--Local Variables----------------------
      REAL*8 SLOPE

      IF (F2*F3.GT.0) THEN
        X = X1 + 0.005D0*(X2 - X1)
        IF (FLAG.EQ.0) THEN
          F = FUNC(X, IPHASE) - Z
        ELSEIF (FLAG.EQ.1) THEN
          F = FUNC(X, Y) - Z
        ELSEIF (FLAG.EQ.2) THEN
          F = FUNC(Y, X) - Z
        ENDIF
        SLOPE = (F - F1)/(X - X1)
        SOLVE2 = X1 - F1/SLOPE
      ELSE
        X = X2 + 0.005D0*(X1 - X2)
        IF (FLAG.EQ.0) THEN
          F = FUNC(X, IPHASE) - Z
        ELSEIF (FLAG.EQ.1) THEN
          F = FUNC(X, Y) - Z
        ELSEIF (FLAG.EQ.2) THEN
          F = FUNC(Y, X) - Z
        ENDIF
        SLOPE = (F - F2)/(X - X2)
        SOLVE2 = X2 - F2/SLOPE
      ENDIF
C     WRITE (*,*) 'X & F:', X, F
C     WRITE (*,*) 'SLOPE:',SLOPE
      END
C=======================================================================
      FUNCTION SOLV1(FUNC, IPHASE, TOL, X1, X2, Z)
C
C  PURPOSE:
C    Find the root, X, of a function of one independent variable,
C    Z = FUNC(X, IPHASE) given a value for Z.  This routine will
C    search X in the interval between the variables X1 and X2.
C
C  DESCRIPTION OF ARGUMENTS:
C    FUNC     A Function of the general form, Y = FUNC(X).      (Input)
C    IPHASE   Switch to select liquid, vapor, or melting line.  (Input)
C    TOL      Converge criteria: ABS((FUNC(X) - Z)/Z) < TOL.    (Input)
C    X1       1st boundary value for the search interval.       (Input)
C    X2       2nd boundary value for the search interval.       (Input)
C    Z        The value of FUNC at the point of interest.       (Input)
C    SOLV1    The value of X after iteration.                   (Output)
C-----------------------------------------------------------------------
      INTEGER IPHASE
      REAL*8 FUNC, SOLV1, SOLVE, TOL, X1, X2, Z
      EXTERNAL FUNC
      SOLV1 = SOLVE(FUNC, IPHASE, 0, TOL, 0.D0, X1, X2, Z)
      END
C=======================================================================
      FUNCTION SOLVX(FUNC, TOL, X1, X2, Y, Z)
C
C  PURPOSE:
C    Find the root, X, of a function of two independent variables,
C    Z = FUNC(X, Y) given values for Z and Y.  This routine will
C    search X in the interval between the variables X1 and X2.
C
C  DESCRIPTION OF ARGUMENTS:
C    FUNC     A Function of the general form, Z = FUNC(X,Y).    (Input)
C    TOL      Converge criteria: ABS((FUNC(X) - Z)/Z) < TOL.    (Input)
C    Y        The value of the known independent variable.      (Input)
C    X1       1st boundary value for the search interval.       (Input)
C    X2       2nd boundary value for the search interval.       (Input)
C    Z        The value of FUNC at the point of interest.       (Input)
C    SOLVX    The value of X after iteration.                   (Output)
C-----------------------------------------------------------------------
      REAL*8 FUNC, SOLVX, SOLVE, TOL, X1, X2, Y, Z
      EXTERNAL FUNC
      SOLVX = SOLVE(FUNC, 0, 1, TOL, Y, X1, X2, Z)
      END
C=======================================================================
      FUNCTION SOLVY(FUNC, TOL, X, Y1, Y2, Z)
C
C  PURPOSE:
C    Find the root, Y, of a function of two independent variables,
C    Z = FUNC(X, Y) given values for Z and X.  This routine will
C    search Y in the interval between the variables Y1 and Y2.
C
C  DESCRIPTION OF ARGUMENTS:
C    FUNC     A Function of the general form, Z = FUNC(X,Y).    (Input)
C    TOL      Converge criteria: ABS((FUNC(X) - Z)/Z) < TOL.    (Input)
C    X        The value of the known independent variable.      (Input)
C    Y1       1st boundary value for the search interval.       (Input)
C    Y2       2nd boundary value for the search interval.       (Input)
C    Z        The value of FUNC at the point of interest.       (Input)
C    SOLVY    The value of Y after iteration.                   (Output)
C-----------------------------------------------------------------------
      REAL*8 FUNC, SOLVY, SOLVE, TOL, Y1, Y2, X, Z
      EXTERNAL FUNC
      SOLVY = SOLVE(FUNC, 0, 2, TOL, X, Y1, Y2, Z)
      END
