function [x_struct,z_struct,u_struct,x0_struct,z0_struct,u0_struct,p_struct,ode,alg] = subsea_model()
    
    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])

    %% Extra functions
    
    xPropane = [0,0,1,0,0,0,0,0,0,0,0]';
    xEthane  = [0,1,0,0,0,0,0,0,0,0,0]';
    xMethane = [1,0,0,0,0,0,0,0,0,0,0]';
    
    g = 9.80665;                            % gravitational constant [m/s2]
    R = 8.31446;                       % universal gas constant [J/(K.mol)]
    
    heatCapacityCoeffs =[...
        1.702,  9.081E-3,  -2.164E-6,       0; ... % C1
        1.131, 19.225E-3,  -5.561E-6,       0; ... % C2
        1.213, 28.785E-3,  -8.824E-6,       0; ... % C3
        1.935, 36.915E-3, -11.402E-6,       0; ... % n-C4
        2.464, 45.351E-3, -14.111E-6,       0; ... % n-C5
        3.025, 53.722E-3, -16.791E-6,       0; ... % n-C6
        3.470,  1.450E-3,          0, 0.121E5; ... % H2O
        5.457,  1.045E-3,          0,-1.157E5; ... % CO2
        3.280,  0.593E-3,          0, 0.040E5; ... % N2
        1.677, 37.853E-3, -11.945E-6,       0; ... % i-C4
        2.464, 45.351E-3, -14.111E-6,       0; ... % i-C5
        ]';

    heatCapacity = @(x,T) ( (heatCapacityCoeffs(1,:)*x)              +  ...
                            (heatCapacityCoeffs(2,:)*x)*(T*1E2)      +  ...
                            (heatCapacityCoeffs(3,:)*x)*(T*1E2)^( 2) +  ...
                            (heatCapacityCoeffs(4,:)*x)*(T*1E2)^(-2)    ...
                          )*R;
    
    % Molar masses [kg/mol]
    molarMassCoeffs =[...
        16.04, ... % C1
        30.07, ... % C2
        44.10, ... % C3
        58.12, ... % n-C4
        72.15, ... % n-C5
        86.18, ... % n-C6
        18.02, ... % H2O
        44.01, ... % CO2
        28.01, ... % N2
        58.12, ... % i-C4
        72.15, ... % i-C5
        ];

    molarMass = @(x) molarMassCoeffs*x;
    
    % Critical temperatures [K]
    criticalTemperatures =[...
        1.90, ... % C1
        3.05, ... % C2
        3.70, ... % C3
        4.25, ... % n-C4
        4.69, ... % n-C5
        5.07, ... % n-C6
        6.47, ... % H2O
        3.04, ... % CO2
        1.26, ... % N2
        4.08, ... % i-C4
        4.60, ... % i-C5
        ];
    
    pseudoCriticalTemperature = @(x) criticalTemperatures*x;

    % Pseudo-reduced temperature [-]
    pseudoReducedTemperature = @(x,T) T/pseudoCriticalTemperature(x);

    % Critical pressures [Pa]
    criticalPressures =[...
        46.0551724137931, ... % C1
        48.8137931034483, ... % C2
        42.5034482758621, ... % C3
        38.0000000000000, ... % n-C4
        33.7241379310345, ... % n-C5
        30.1379310344828, ... % n-C6
        220.620689655172, ... % H2O
        73.8344827586207, ... % CO2
        33.9000000000000, ... % N2
        36.4896551724138, ... % i-C4
        33.9000000000000, ... % i-C5
        ];

    % Pseudo-critical mixture pressure [Pa]
    pseudoCriticalPressure = @(x) criticalPressures*x;

    % Pseudo reduced pressure [-]
    pseudoReducedPressure = @(x,P) P/pseudoCriticalPressure(x);

    % Compressibility factor
    %
    %   Calculated using Dranchuk and Abou-Kassem EOS
    %   P.M. Dranchuk and H. Abou-Kassem - " Calculation of Z Factors For 
    %   Natural Gases Using Equations of State" (1975)

    A1 = 0.3265;
    A2 = -1.0700;
    A3 = -0.5339;
    A4 = 0.01569;
    A5 = -0.05165;
    A6 = 0.5475;
    A7 = -0.7361;
    A8 = 0.1844;
    A9 = 0.1056;
    A10 = 0.6134;
    A11 = 0.7210;
    
    Tpr = pseudoReducedTemperature;
    Ppr = pseudoReducedPressure;
    
    compressibilityFactor = @(x,T,P,Z) ...
        1+(A1+A2/Tpr(x,T)+A3/Tpr(x,T)^3+A4/Tpr(x,T)^4+A5/Tpr(x,T)^5)*(0.27*Ppr(x,P)/(Z*Tpr(x,T))) + ...
          (A6+A7/Tpr(x,T)+A8/Tpr(x,T)^2)*(0.27*Ppr(x,P)/(Z*Tpr(x,T)))^2 - ...
          (A7/Tpr(x,T)+A8/Tpr(x,T)^2)*A9*(0.27*Ppr(x,P)/(Z*Tpr(x,T)))^5 + ...
           A10*(1+A11*(0.27*Ppr(x,P)/(Z*Tpr(x,T)))^2)*((0.27*Ppr(x,P)/(Z*Tpr(x,T)))^2/Tpr(x,T)^3)*exp(-A11*(0.27*Ppr(x,P)/(Z*Tpr(x,T)))^2);
                      
                      
    %% Define the streams
    
    M1 = molarMass(xPropane);
    
    z_struct = struct();
    u_struct = struct();
    
    T1 = MX.sym('T1');                                    % Temperature [K]
    P1 = MX.sym('P1');                                      % Pressure [Pa]
    m1 = MX.sym('m1');                            % Molar flow rate [mol/s]   
    Cp1 = MX.sym('Cp1');                        % Heat capacity [J/(mol.K)]  
    Z1 = MX.sym('Z1');
    
    T2 = MX.sym('T2');                                    % Temperature [K]
    P2 = MX.sym('P2');                                      % Pressure [Pa]
    m2 = MX.sym('m2');                            % Molar flow rate [mol/s]
    Cp2 = MX.sym('Cp2');                        % Heat capacity [J/(mol.K)] 
    
    N12 = MX.sym('N12');                    % average adiabatic ratio [-]
    gamma12 = MX.sym('gamma12');            % average adiabatic ratio [-]
    k12     = MX.sym('k12');                            % k = n/(n+1) [-] 
    nu12    = MX.sym('nu12');                 % compressor efficiency [%]
    H12     = MX.sym('H12');                        % compressor head [m]
    %Pow12   = MX.sym('Pow12');         % compressor power consumption [W]
    %srg12   = MX.sym('srg12');                      % indicates surge [-]
    %stw12   = MX.sym('stw12');          % indicates Stonewall / choke [-]
    
    z_struct.T1 = T1;
    z_struct.P1 = P1;
    z_struct.m1 = m1;
    z_struct.Cp1 = Cp1;
    z_struct.Z1 = Z1;
    z_struct.T2 = T2;
    z_struct.P2 = P2;
    z_struct.m2 = m2;
    z_struct.Cp2 = Cp2;
    z_struct.gamma12 = gamma12;
    z_struct.k12 = k12;
    z_struct.nu12 = nu12;
    z_struct.H12 = H12;
    
    u_struct.N12 = N12;
    
    % Define equations 
    algs = struct();

    algs.Cp1 = Cp1 - heatCapacity(xPropane,T1);
    algs.Z1 = Z1 - compressibilityFactor(xPropane,T1,P1,Z1);
    
    algs.m2  = m1 - m2;                             % Mass balance m1 and m2
    algs.Cp2 = Cp2 - heatCapacity(xPropane,T2);
    
    algs.P2 = T2/T1 - (P2/P1)^(1/k12);
    algs.k12 = k12  - nu12*gamma12/(gamma12-1);
    algs.gamma12 = gamma12 - 1/2*( Cp1/(Cp1-R) + Cp2/(Cp2-R));
    
    qN = (m1/18.75*1.5)/N12;
    
    c1 = 0.582;                          % parameters for the fit nu = f(q)
    c2 = -2.398;
    c3 = 2.75;
    c4 = - 3.969;
    c5 = 4.303;
    
    algs.nu12 = nu12 - (c1*qN^2 + c2*qN + c3)/(qN^2 + c4*qN + c5);

    c6 = 5.779832328060393;              % parameters for the fit H = f(q)
    c7 = -1.851667235689085;
    c8 =  1.081537119229164;
    
    algs.T2  = H12*1E-2  - k12*Z1*R/(g*M1)*(T2-T1);
    
    
    Ps = [];
    for i=linspace(10,20,100)
    algs.H12 = H12  -  i; %(c6*qN^2 + c7*qN + c8)*N12^2;
    
    algs.m1 = m1 - 18.75;
    algs.T1 = T1 - (298.15-41.99)/100;
    algs.P1 = P1 - 1.022;
    algs.N12 = N12 - 0.90;
    
    rf_function = Function('rf_function',{vertcat(N12,casadi_struct2vec(z_struct))},{casadi_struct2vec(algs)});
    
    %algs.Pow = (Pow*1E7) - (H*1E3)*q1*rho1*g/nu;
    %algs.srg = srg - (qN-qmin);
    %algs.stw = stw - (qmax-qN);
    
    rf = rootfinder('rf','kinsol',rf_function);

    z0 = [0.90,...
          2.56,...
          1.022,...
          18.75,...
          67,...
          0.96,...
          3,...
          10,...
          18.75,...
          80,...
          1.2,...
          7,...
          0.8,...
          12 ...
          ]';
    z0 = full(rf(z0))
    
    Ps = [Ps z0(8)];
    end
   
end