function [comp,outlet] = compressor(namein,nameout,inlet,Ni,hi)

    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*  
    
    nm = num2str(namein);
    
    outlet = def_stream(nameout,inlet.p.x,false,[],[],[],[]);
    
    N = MX.sym('N');                      % normalized compressor speed [%]
    h = MX.sym('h');                                % compressor health [-]
    
    gamma = MX.sym(['gamma_',nm]);            % average adiabatic ratio [-]
    k     = MX.sym(['k_',nm]);                            % k = n/(n+1) [-] 
    nu    = MX.sym(['nu_',nm]);                 % compressor efficiency [%]
    H     = MX.sym(['H_',nm]);                        % compressor head [m]
    Pow   = MX.sym(['Pow_',nm]);         % compressor power consumption [W]
    srg   = MX.sym(['srg_',nm]);                      % indicates surge [-]
    stw   = MX.sym(['stw_',nm]);          % indicates Stonewall / choke [-]
    Paris = MX.sym(['Paris_',nm]);
    a     = MX.sym(['a_',nm]);                             % wear parameter
    
    
    z = struct();                                     % algebraic variables
    x = struct();                                  % differential variables
    u = struct();                                                  % inputs
    p = struct();                                              % parameters
    
    algs = struct();        % Equations for residuals / algebraic equations
    odes = struct();      % Equations for dynamics / differential equations
    
    u.N     = N;
    x.h     = h;
    z.gamma = gamma;
    z.k     = k;
    z.nu    = nu;
    z.H     = H;
    z.Pow   = Pow;
    z.srg   = srg;
    z.stw   = stw;
    z.Paris = Paris;
    p.a     = a;
    
    % Extract variables from streams
    T1    = inlet.z.T                         *1E2;
    T2    = outlet.z.T                        *1E2;
    P1    = inlet.z.P                         *1E7;
    P2    = outlet.z.P                        *1E7;
    Cp1   = inlet.z.Cp                        *1E2;
    Cp2   = outlet.z.Cp                       *1E2;
    q1    = (inlet.z.Vgdot+inlet.z.Vcdot);
    M1    = inlet.p.M                         *1E-3;
    Z1    = inlet.z.Z;
    rho1  = inlet.z.rho                       *1E2;
    GVF1  = inlet.z.GVF;
    
    % Parameters
    g     = 9.80665;                        % gravitational constant [m/s2]
    R     = 8.31446;                   % universal gas constant [J/(K.mol)]
    rho_l = 10;                                         % Density of liquid
    
    c1 = 0.582;                          % parameters for the fit nu = f(q)
    c2 = -2.398;
    c3 = 2.75;
    c4 = - 3.969;
    c5 = 4.303;
    
    c6 = -0.9937;                         % parameters for the fit H = f(q)
    c7 = 2.256;
    c8 = 1.888;
    
    qmin = 1.163;              % Minimum allowable flow at N=1 (surge line)
    qmax = 2.286;              % Maximum allowable flow at N=1 (choke line)
    
    %% Define the residuals
    head_par = 4;                               % Head adjustment parameter
    qN       = q1/N;                 % Flow corresponding to N=1 (fan laws)
    rho_avg1 = GVF1*rho1+(1-GVF1)*rho_l;                  % Average density
    f_wood   = 1/(rho_avg1/rho1*sqrt(rho_avg1/rho1*GVF1)); % Woods cor.fac.
    
    algs.nu = nu - (c1*qN^2 + c2*qN + c3)/(qN^2 + c4*qN + c5);
    algs.T  = (H*1E3)  - k*Z1*R/(g*M1)*(T2-T1);
    algs.H  = H  - head_par*(c6*qN^2 + c7*qN + c8)*N^2*f_wood;
    algs.k  = k  - nu*gamma/(gamma-1);
    algs.gamma = gamma - 1/2*( Cp1/(Cp1-R) + Cp2/(Cp2-R));
    algs.P  = T2/T1 - (P2/P1)^(1/k);
    
    algs.Pow = (Pow*1E7) - (H*1E3)*q1*rho1*g/nu;
    algs.srg = srg - (qN-qmin);
    algs.stw = stw - (qmax-qN);
    algs.mg = inlet.z.mgdot - outlet.z.mgdot;
    algs.mc = inlet.z.mcdot - outlet.z.mcdot;
    algs.Paris = Paris - Pow^2/N; 
    
    %% Define the dynamics
    odes.dh = a*2.7*Paris*h;
    
    %% Collect the variables and equations into struct
    comp = struct();
    comp.z = z;
    comp.x = x;
    comp.u = u;
    comp.p = p;
    comp.algs = algs;
    comp.odes = odes;
    
    %% Initialize
    test = [N-Ni;h-hi;casadi_struct2vec(inlet.z)-inlet.z0];
    rf_function = Function('rf_function',...
        {vertcat(casadi_struct2vec(inlet.z),casadi_struct2vec(outlet.z),casadi_struct2vec(z),N,h)},...
        {vertcat(test,casadi_struct2vec(outlet.algs),casadi_struct2vec(algs))});

    rf = rootfinder('rf','kinsol',rf_function);
    
    z0in = inlet.z0;
    z0out = z0in; z0out(2) = z0out(2)*1.4;
    
    guess = [1.2,4,0.77,9,0.96,0.4,075,1.1]';
    
    z0        = full(rf(vertcat(z0in,z0out,guess,Ni,hi)));
    outlet.z0 = z0(length(inlet.z0)+1:2*length(inlet.z0));
    comp.z0   = z0(2*length(inlet.z0)+1:end-2);
end