function stream = def_stream(name,x,init,Ti,Pi,mgdoti,mcdoti)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% Returns a struct containing                                             %
%                                                                         %
%   vars: decision variables: T,P,mdot,x, M,Z,Cp,rho,Vdot                 %
%   resi: equations for the residuals of M,Z,Cp,rho,Vdot                  %
%                                                                         %
% name: name of stream                                                    %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
import casadi.* 

nm = num2str(name);                                                        % SCALING FACTORS                 
T = MX.sym(['T_',nm]);                                    % Temperature [K]	-2
P = MX.sym(['P_',nm]);                                      % Pressure [Pa]	-7
mgdot = MX.sym(['mgdot_',nm]);                      % Mass flow rate [kg/s]	-2    
mcdot = MX.sym(['mcdot_',nm]);                      % Mass flow rate [kg/s]	-2   

z = struct();                                          % Decision variables
p = struct();                                            % Extra parameters
algs = struct();                                  % Equations for residuals

%% Declare variables

Z     = MX.sym(['Z_',nm]);                            % Compressibility [-]  0           
Cp    = MX.sym(['Cp_',nm]);                     % Heat capacity [J/(mol.K)] -2
rho   = MX.sym(['rho_',nm]);                       % Density of gas [kg/m3] -2
Vgdot = MX.sym(['Vgdot_',nm]);         % Volumetric flow rate of gas [m3/s]  0
Vcdot = MX.sym(['Vcdot_',nm]);  % Volumetric flow rate of condensate [m3/s]  0
GVF   = MX.sym(['GVF_',nm]);                      % Gas-volume-fraction [-]	 0

%% Concatenate variables

z.T     = T;
z.P     = P;
z.mgdot = mgdot;
z.mcdot = mcdot;

z.Z     = Z;
z.Cp    = Cp;
z.rho   = rho;
z.Vgdot = Vgdot;
z.Vcdot = Vcdot;
z.GVF   = GVF;

%% Define properties

% Molar masses [kg/mol]
M_ =[...
    16.04, ... % C1
    30.07, ... % C2
    44.10, ... % C3
    58.12, ... % n-C4
    72.15, ... % n-C5
    86.18, ... % n-C6
    18.02, ... % H2O
    44.01, ... % CO2
    28.01, ... % N2
    58.12, ... % i-C4
    72.15, ... % i-C5
    ];

M = M_*x';
p.M = M;
p.x = x;

% Critical temperatures [K]
Tc_ =[...
    190, ... % C1
    305, ... % C2
    370, ... % C3
    425, ... % n-C4
    469, ... % n-C5
    507, ... % n-C6
    647, ... % H2O
    304, ... % CO2
    126, ... % N2
    408, ... % i-C4
    460, ... % i-C5
    ];

% Pseudo-critical mixture temperature [K]
%
%   Calculated from Kay's rule:
%   W.B. Kay - "Gases and Vapors At High Temperature and Pressure - Density
%   of Hydrocarbon" (1936)
%
%   Known to be innaccurate, see for instance 
%   R.P. Sutton - "Compressibility Factors for High-Molecular-Weight 
%   Reservoir Gases" (1985)

Tpc = Tc_*x';

% Pseudo-reduced temperature [-]
Tpr = (T*1E2) / Tpc;

% Critical pressures [Pa]
Pc_ =[...
    4.60551724137931E6, ... % C1
    4.88137931034483E6, ... % C2
    4.25034482758621E6, ... % C3
    3.80000000000000E6, ... % n-C4
    3.37241379310345E6, ... % n-C5
    3.01379310344828E6, ... % n-C6
    22.0620689655172E6, ... % H2O
    7.38344827586207E6, ... % CO2
    3.39000000000000E6, ... % N2
    3.64896551724138E6, ... % i-C4
    3.39000000000000E6, ... % i-C5
    ];

% Pseudo-critical mixture pressure [Pa]
Ppc = Pc_*x';

% Pseudo reduced pressure [-]
Ppr = (P*1E7) / Ppc;

% Compressibility factor
%
%   Calculated using Dranchuk and Abou-Kassem EOS
%   P.M. Dranchuk and H. Abou-Kassem - " Calculation of Z Factors For 
%   Natural Gases Using Equations of State" (1975)

A1 = 0.3265;
A2 = -1.0700;
A3 = -0.5339;
A4 = 0.01569;
A5 = -0.05165;
A6 = 0.5475;
A7 = -0.7361;
A8 = 0.1844;
A9 = 0.1056;
A10 = 0.6134;
A11 = 0.7210;

tmp = 0.27*Ppr/(Z*Tpr);

algs.Z = -Z + 1+(A1+A2/Tpr+A3/Tpr^3+A4/Tpr^4+A5/Tpr^5)*tmp + ...
      (A6+A7/Tpr+A8/Tpr^2)*tmp^2 - ...
      (A7/Tpr+A8/Tpr^2)*A9*tmp^5 + ...
       A10*(1+A11*tmp^2)*(tmp^2/Tpr^3)*exp(-A11*tmp^2);

% Heat capacity 
%
%   Coefficients  from: 
%   "http://www.personal.utulsa.edu/~geoffrey-price/Courses/ChE7023/HeatCapacity-HeatOfFormation.pdf

coeffs =[...
    1.702,  9.081E-3,  -2.164E-6,       0; ... % C1
    1.131, 19.225E-3,  -5.561E-6,       0; ... % C2
    1.213, 28.785E-3,  -8.824E-6,       0; ... % C3
    1.935, 36.915E-3, -11.402E-6,       0; ... % n-C4
    2.464, 45.351E-3, -14.111E-6,       0; ... % n-C5
    3.025, 53.722E-3, -16.791E-6,       0; ... % n-C6
    3.470,  1.450E-3,          0, 0.121E5; ... % H2O
    5.457,  1.045E-3,          0,-1.157E5; ... % CO2
    3.280,  0.593E-3,          0, 0.040E5; ... % N2
    1.677, 37.853E-3, -11.945E-6,       0; ... % i-C4
    2.464, 45.351E-3, -14.111E-6,       0; ... % i-C5
    ];

c = x*coeffs;
R = 8.31446;
algs.Cp = -(Cp*1E2) + (c(1) + c(2)*(T*1E2) + c(3)*(T*1E2)^2 + c(4)*(T*1E2)^(-2))*R;

% Density
algs.rho = -(rho*1E2) + (P*1E7)*(M*1E-3)/(8.3144*Z*(T*1E2));

% Volumetric flow rate
rho_condensate = 10;
algs.Vgdot = -(mgdot*1E2) + (Vgdot)*(rho*1E2);
algs.Vcdot = -(mcdot*1E2) + (Vcdot)*(rho_condensate*1E2);

% GVF
algs.GVF = -GVF + Vgdot/(Vgdot+Vcdot);

%% Define stream object

stream      =  struct();
stream.z    =  z;
stream.p    =  p;
stream.algs =  algs;

if init

%% Initialize
Zi = 0.89;
Cpi = 40;
rhoi = 68;
Vgdoti = 179.743;
Vcdoti = 19.97;
GVFi = 0.9;
test = [T-Ti;P-Pi;mgdot-mgdoti;mcdot-mcdoti];
rf_function = Function('rf_function',{casadi_struct2vec(z)},{vertcat(test,casadi_struct2vec(algs))});

rf = rootfinder('rf','kinsol',rf_function);

z0 = [Ti,Pi,mgdoti,mcdoti,Zi,Cpi,rhoi,Vgdoti,Vcdoti,GVFi]';
stream.z0 = full(rf(z0));

end
%keyboard