clear

% Provide path to Casadi installation
addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
import casadi.*

N  = 20;
dt = 1/N;

h0=0.01;

[dae_x,dae_z,dae_u,~,dae_z0,dae_u0,dae_p,dae_ode,dae_alg] = subsea_model();    

hlist = (1/N)*ones(1,N);
tf = sum(hlist);

% Stochastic variables: GVF and degradation speed
dae_stoc.a     = dae_p.a;
dae_stoc.GVF   = dae_z.stream1.GVF;
dae_z.stream1  = rmfield(dae_z.stream1,'GVF');
dae_z0.stream1 = rmfield(dae_z0.stream1,'GVF');

% Cost function
quad = -(dae_z.stream4.mgdot)';

dae      = struct;
dae.x    = casadi_struct2vec(dae_x);
dae.z    = casadi_struct2vec(dae_z);
dae.p    = casadi_struct2vec(dae_u);
dae.s    = casadi_struct2vec(dae_stoc);
dae.ode  = casadi_struct2vec(dae_ode);
dae.alg  = dae_alg;
dae.quad = quad;

nx = size(dae.x,1);
nu = size(dae.p,1);
nz = size(dae.z,1);
ns = size(dae.s,1);

% Degree of interpolating polynomial
d = 3;

% Get collocation points
tau_root = casadi.collocation_points(d, 'radau');
collfun = simpleColl(dae,tau_root);

nlp = {};
for scen=1:3

    X0s = MX.sym('X0',nx);

    Xs = cell(N+1,1);
    for i=1:N+1
        Xs{i} = MX.sym(['X_' num2str(i)],nx);
    end
    XCs = cell(N,1);
    Zs  = cell(N,1);
    Us  = cell(N,1);
    Ss  = cell(N,1);

    for i=1:N
        XCs{i} = MX.sym(['XC_' num2str(i)],nx,d);
        Zs{i}  = MX.sym(['Z_'  num2str(i)],nz,d);
        Us{i}  = MX.sym(['U_'  num2str(i)],nu,1);
        Ss{i}  = MX.sym(['S_'  num2str(i)],ns,1);
    end

    V_block = struct();
    V_block.X  = Sparsity.dense(nx,1);
    V_block.XC = Sparsity.dense(nx,d);
    V_block.Z  = Sparsity.dense(nz,d);
    V_block.U  = Sparsity.dense(nu,1);

    % Bounds on states and constraints
    lbx = {};
    ubx = {};
    lbg = {};
    ubg = {};

    % Objective function
    f = 0;

    % List of constraints
    g = {};

    % List of all decision variables (determines ordering)
    V    = {};

    %% Define bounds

    % Bounds at 0<t<tf
    x_lb_k = casadi_vec(dae_x,0.0);
    x_ub_k = casadi_vec(dae_x,1.0);
    u_lb_k = casadi_vec(dae_u,0,'N',0.75,'Z',0);
    u_ub_k = casadi_vec(dae_u, inf,'N',1.05,'Z',1);
    z_lb_k = casadi_vec(dae_z, -inf,'srg',0,'stw',0,'stream4',{'P',1.5});
    z_ub_k = casadi_vec(dae_z, inf);
    z_lb_k = repmat(z_lb_k',d)';        % repeat for each collocation point
    z_ub_k = repmat(z_ub_k',d)';

    lbx_k  = {casadi_vec(V_block,  0,'X',x_lb_k,'U',u_lb_k,'Z',z_lb_k)};
    ubx_k  = {casadi_vec(V_block,inf,'X',x_ub_k,'U',u_ub_k,'Z',z_ub_k)};

    % Bounds at t=tf
    x_lb_tf = {casadi_vec(dae_x,0.0)};
    x_ub_tf = {casadi_vec(dae_x,1.0)};

    guess = [];

    z_guess   = casadi_struct2vec(dae_z0);
    u_guess   = casadi_struct2vec(dae_u0);

    for k=1:N
        % Add decision variables
        V = [V {casadi_vec(V_block,'X',Xs{k},'XC',XCs{k},               ...
                                   'Z',Zs{k},'U',Us{k})}];

        guess = [guess;[repmat(0.1*exp(2.1*k/N),d+1,1);                 ...
                repmat(z_guess,d,1);u_guess]];

        lbx = [lbx lbx_k];
        ubx = [ubx ubx_k];

        if k==1
            tmp = {Xs{k}-X0s};
            g   = [g tmp]; 
            lbg = [lbg {zeros(size(tmp{:}))}];
            ubg = [ubg {zeros(size(tmp{:}))}];

            % Enforce nonanticipativity
            if scen==1
                U1 = Us{1};
            else
                tmp = {Us{k}-U1};
                g = [g tmp];
                lbg = [lbg {zeros(size(tmp{:}))}];
                ubg = [ubg {zeros(size(tmp{:}))}];
            end
        end

        % Obtain collocation expressions
        h = hlist(k);
        coll_out = collfun.call({h,Xs{k},XCs{k},Zs{k},Us{k},Ss{k}});

        tmp = coll_out(2);
        g   = [g tmp];                                    % System dynamics
        lbg = [lbg {zeros(size(tmp{:}))}];
        ubg = [ubg {zeros(size(tmp{:}))}];

        tmp = coll_out(3);
        g   = [g tmp];                              % Algebraic constraints
        lbg = [lbg {zeros(size(tmp{:}))}];
        ubg = [ubg {zeros(size(tmp{:}))}];

        tmp = {Xs{k+1}-coll_out{1}};
        g   = [g tmp];                            % Gap closing constraints
        lbg = [lbg {zeros(size(tmp{:}))}];
        ubg = [ubg {zeros(size(tmp{:}))}];
        
                            % Cost function: future flows are discounted to 
                                                 % obtain net present value
        f = f + (coll_out{4}*(hlist(k)*(1+0.015)^(-60*sum(hlist(1:k)))));
    end

    % Add final x to decision variables
    V = [V,Xs(end)];
    guess = [guess;1];

    % Bounds for final t
    lbx = [lbx x_lb_tf];
    ubx = [ubx x_ub_tf];

    %% Define the NLP
    nlp{scen} = struct('x',vertcat(V{:}),                               ...
                       'f',f,                                           ...
                       'g',vertcat(g{:}),                               ...
                       'p',vertcat(X0s,Ss{:})                           ...
                       );

end         

% Generate the full-space NLP by combining the individual scenario problems
nlp_full_space = nlp{1};
for i = 2:scen
    nlp_full_space.x = horzcat(nlp_full_space.x,nlp{i}.x);
    nlp_full_space.g = vertcat(nlp_full_space.g,nlp{i}.g);
    nlp_full_space.p = vertcat(nlp_full_space.p,nlp{i}.p);
    nlp_full_space.f = nlp_full_space.f + nlp{i}.f;
end

nlpfun = Function('nlp',nlp_full_space,char('x','p'),char('f','g'));

opts = struct('warn_initial_bounds',false,                              ...
              'gather_stats',true,                                      ...
              'print_time',false,                                       ...
              'ipopt',struct('linear_solver','mumps',                   ...
                             'max_iter',5E2,                            ...
                             'warm_start_init_point','yes',             ...
                             'mu_init',1E-5,                            ...
                             'replace_bounds','yes',                    ...
                             'print_level',5)                           ...
              ); 
         
solver = nlpsol('solver','ipopt',nlp_full_space,opts);

% Scenario realizations
b1 = [0.9,1];
b2 = [1.0,1];
b3 = [1.1,1];

s11 = [b1;repmat(b1,N-1,1)]';
s22 = [b2;repmat(b2,N-1,1)]';
s33 = [b3;repmat(b3,N-1,1)]';

%% Solve the NLP
res = solver('x0',repmat(guess(:),1,scen),                              ...
             'p',[h0 ; s11(:) ; h0 ; s22(:) ; h0 ; s33(:)],             ...
             'lbg',zeros(size(nlp_full_space.g)),                       ...
             'ubg',zeros(size(nlp_full_space.g)),                       ...
             'lbx',repmat(vertcat(lbx{:}),1,scen),                      ...
             'ubx',repmat(vertcat(ubx{:}),1,scen)                       ...
             );       

%% Plotting

close all
figure('units','normalized','outerposition',[0 0 1 1])
hold on

for i = 1:scen
    
    % Which variables to plot
    vars_Z = {'stream4.P','stream4.mgdot','compressor.Pow',...
              'compressor.Paris','compressor.srg','compressor.nu',...
              'compressor.stw','stream3.Vgdot','stream3.Vcdot',...
              'stream3.GVF','stream3.Z'}; 
    vars_X = {'compressor.h'};
    vars_U = {'compressor.N','choke.Z'};

    dim = size(casadi_struct2vec(V_block));
    tmp = DM(reshape(full(res.x),[res.x.size1(),scen]));
    res_split = vertsplit(tmp,dim(1));

    while true % plotting

        n_plots = length(vars_Z)+length(vars_X)+length(vars_U);
        flr = floor(sqrt(n_plots));
        cei = ceil(sqrt(n_plots));
        if flr==sqrt(n_plots)
            plot_dims = [flr,flr];
        elseif flr*cei>=n_plots
            plot_dims = [flr,cei];
        else
            plot_dims = [cei,cei];
        end

        counter = 1;

        res_Z  = {};
        res_X  = {};
        res_XC = {};
        res_U  = {};

        for r = res_split(1:end-1)
            r = full(r{:});
            r = r(:,i);
            rs = casadi_vec2struct(V_block,r(:));
            res_Z  = [res_Z  {rs.Z} ];
            res_X  = [res_X  {rs.X} ];
            res_XC = [res_XC {rs.XC}];
            res_U  = [res_U  {rs.U} ];
        end
        res_Z  = full([res_Z{:}])';
        res_X  = full([res_X{:}])';
        res_XC = full([res_XC{:}])';
        res_U  = full([res_U{:}])';

        %% Plot Z
        [nr,nc] = size(res_Z);
        mat_Z = mat2cell(res_Z,[nr],ones(nc,1));
        indices_Z = casadi_vec2struct(dae_z,1:nc);

        t_z = [];
        tmp = N*(cumsum([0 hlist(1:end-1)]));
        for t=1:length(tmp); t_z =[t_z tmp(t)+tau_root*hlist(t)*N]; end
        t_z = t_z'/N;

        z_lb = full(z_lb_k(:,1));
        z_ub = full(z_ub_k(:,1));
        for j = 1:length(vars_Z)
            subplot(plot_dims(1),plot_dims(2),counter)
            counter = counter+1;
            hold on
            str = strsplit(vars_Z{j},'.');      
            index = full(indices_Z.(str{1}).(str{2}));
            plot(t_z,ones(size(t_z)).*z_lb(index),'r--')
            plot(t_z,ones(size(t_z)).*z_ub(index),'r--')
            plot(t_z,mat_Z{index})
            xlim([0,1])
            title(vars_Z(j))
            xlabel('Time')
            ylabel(str{2})
        end

        %% Plot X
        [nr,nc] = size(res_XC);
        mat_XC = mat2cell(res_XC,[nr],ones(nc,1));
        indices_XC = casadi_vec2struct(dae_x,1:nc);

        t_xc = [];
        tmp = N*(cumsum([0 hlist(1:end-1)]));
        for t=1:length(tmp); t_xc =[t_xc tmp(t)+tau_root*hlist(t)*N]; end
        t_xc = t_xc'/N;

        xc_lb = full(x_lb_k(:,1));
        xc_ub = full(x_ub_k(:,1));
        for i = 1:length(vars_X)
            subplot(plot_dims(1),plot_dims(2),counter)
            counter = counter+1;
            hold on
            str = strsplit(vars_X{i},'.');
            index = full(indices_XC.(str{1}).(str{2}));
            plot(t_xc,ones(size(t_xc)).*xc_lb(index),'r--')
            plot(t_xc,ones(size(t_xc)).*xc_ub(index),'r--')
            plot(t_xc,mat_XC{index})
            xlim([0,1])
            title(vars_X(i))
            xlabel('Time')
            ylabel(str{2})
        end

        %% Plot U
        [nr,nc] = size(res_U);
        mat_U = mat2cell(res_U,[nr],ones(nc,1));
        indices_U = casadi_vec2struct(dae_u,1:nc);

        t_u = N*(cumsum([0 hlist(1:end-1)]));
        t_u = t_u'/N;

        u_lb = full(u_lb_k(:,1));
        u_ub = full(u_ub_k(:,1));
        for i = 1:length(vars_U)
            subplot(plot_dims(1),plot_dims(2),counter)
            counter = counter+1;
            hold on
            str = strsplit(vars_U{i},'.');
            index = full(indices_U.(str{1}).(str{2}));
            plot(t_u,ones(size(t_u)).*u_lb(index),'r--')
            plot(t_u,ones(size(t_u)).*u_ub(index),'r--')
            stairs(t_u,mat_U{index})
            xlim([0,1])
            title(vars_U(i))
            xlabel('Time')
            ylabel(str{2})
        end

        hold off
        break
    end

end
