function [x_struct,z_struct,u_struct,x0_struct,z0_struct,u0_struct,p_struct,ode,alg] = subsea_model()

    %
    %                                      comp                |
    %                         stream3 _____|\____ stream4     | 
    %                                |     |_/    |            |
    %                              /\          |            |
    %                              |   |          |            |
    %             _________________|sep|          |____________/
    %             |                |   |          | 
    %          stream2             \___/          |
    %            _|_                 |_____|\____| 
    %            \V/          stream5      |_/         
    %      choke /A\                      (pump)
    %            |
    %             |
    %          stream1 
    %
    
    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])

    %% Define the streams
    
    x = [.92,.05,.02,.005,.005,0,0,0,0,0,0];
    T1 = 3.5;
    P1 = 1.0;
    mgdot1 = 0.9;%0.8997;
    mcdot1 = 0;%1.4616;
    u_choke = 0.565;
    u_comp = 0.85;
    xi = 0.01;
    
    % First well
    stream1 = def_stream('1',x,true,T1,P1,mgdot1,mcdot1);

    % Choke
    [chk,stream2] = choke('chk','2',stream1,u_choke);                      
    
    % Separator
    [sep,stream3,stream5] = separator('sep','3','5',stream2);
    
    % Compressor
    [comp,stream4] = compressor('comp34','4',stream3,u_comp,xi);
    
    %% Concatenate the decision variables

    % Algebraic variables
    z_struct            = struct();
    z_struct.stream1    = rmfield(stream1.z,                            ...
                                {'Cp'});
    z_struct.stream2    = rmfield(stream2.z,                            ...
                                {'Cp','Vcdot','GVF'});
    z_struct.stream3    = rmfield(stream3.z,                            ...
                                {});
    z_struct.stream4    = rmfield(stream4.z,                            ...
                                {'rho','Vgdot','Vcdot','Z','GVF'});
    z_struct.stream5    = rmfield(stream5.z,                            ...
                                {'Cp','rho','Vgdot','Vcdot','Z','GVF'});
    z_struct.separator  = sep.z;
    z_struct.compressor = comp.z;
    
    z0_struct            = struct();
    z0_struct.stream1    = rmfield(casadi_vec2struct(stream1.z,         ...
                                 stream1.z0),                           ...
                                 {'Cp'}                                 ...
                                 );
    z0_struct.stream2    = rmfield(casadi_vec2struct(stream2.z,         ...
                                 stream2.z0),                           ...
                                 {'Cp','Vcdot','GVF'}                   ...
                                 );
    z0_struct.stream3    = rmfield(casadi_vec2struct(stream3.z,         ...
                                stream3.z0),                            ...
                                {}                                      ...
                                );
    z0_struct.stream4    = rmfield(casadi_vec2struct(stream4.z,         ...
                                stream4.z0),                            ...
                                {'rho','Vgdot','Vcdot','Z','GVF'}       ...
                                );
    z0_struct.stream5    = rmfield(casadi_vec2struct(stream5.z,         ...
                                stream5.z0),                            ...
                                {'Cp','rho','Vgdot','Vcdot','Z','GVF'}  ...
                                );
    z0_struct.separator  = sep.z0;
    z0_struct.compressor = comp.z0;

    % Differential variables
    x_struct = struct();
    x_struct.compressor = comp.x;
    
    x0_struct = struct();
    x0_struct.compressor = 0.01;

    % Inputs
    u_struct = struct();
    u_struct.compressor = comp.u;
    u_struct.choke      = chk.u;
    
    u0_struct = struct();
    u0_struct.compressor = 1.05;
    u0_struct.choke      = 0.95;
    
    % Parameters
    p_struct = struct();
    p_struct.a = comp.p.a;
    p_struct.GVF = stream1.z.GVF;
    
    %% Define equations

    % Algebraic equations
    sep_algs = sep.algs;
    sep_algs.gas = rmfield(sep_algs.gas,{'T','P'});
    sep_algs.liq = rmfield(sep_algs.liq,{'T','P'});

    alg = [                                                             ...
           casadi_struct2vec(rmfield(stream1.algs,                      ...
                             {'Cp'}                                     ...
                             ));                                        ...
           casadi_struct2vec(rmfield(stream2.algs,                      ...
                             {'Cp','Vcdot','GVF'}                       ...
                             ));                                        ...
           casadi_struct2vec(rmfield(stream3.algs,                      ...
                             {}                                         ...
                             ));                                        ...
           casadi_struct2vec(rmfield(stream4.algs,                      ...
                             {'rho','Vgdot','Vcdot','Z','GVF'}          ...
                             ));                                        ...
           casadi_struct2vec(rmfield(stream5.algs,                      ...
                             {'Cp','rho','Vgdot','Vcdot','Z','GVF'}     ...
                             ));                                        ...
           casadi_struct2vec(rmfield(chk.algs,                          ...
                             {'T'}                                      ...
                             ));                                        ...
           casadi_struct2vec(sep_algs);                                 ...
           casadi_struct2vec(rmfield(comp.algs,{}));                    ...
          ];

    % Differential equations

    ode = [                                                             ...
           casadi_struct2vec(comp.odes);                                ...
          ];
      
    % Remove unneeded variables (to reduce size of system)
    s1 = {...                                         % Boundary conditions
          {'stream1','T'},                                              ...
          {'stream2','T'},                                              ...
          {'stream3','T'},                                              ... 
          {'stream5','T'},                                              ... 
          {'stream1','P'},                                              ...
          {'stream2','P'},                                              ...
          {'stream3','P'},                                              ...
          {'stream5','P'},                                              ...
         };                         

    v1 = {...
          3.5,                                                          ...
          3.5,                                                          ...
          3.5,                                                          ...
          3.5,                                                          ...
          1.0,                                                          ...
           .9,                                                          ...
           .9,                                                          ...
           .9,                                                          ...
         };

    for i=1:length(s1)
        cell = s1{i};
        z                   = z_struct.(cell{1}).(cell{2});
        z_struct.(cell{1})  = rmfield(z_struct.(cell{1}),cell{2});
        z0_struct.(cell{1}) = rmfield(z0_struct.(cell{1}),cell{2});
        alg = substitute(alg ,z,v1{i});
        ode = substitute(ode ,z,v1{i});
    end
end